from v_cloud_market_cli_common.utils.server_api_wrapper import NewServerWrapper
from v_cloud_market_cli_common.config.server_config import API_VERSION
from .service_common import ServiceCommon

class UserServiceQueryParam:
    service_active = True
    service_id = ''
    service = ''
    status = []
    start_from = 0
    end_at = 0
    current = 1
    page_size = 10

    def as_dict(self) -> dict:

        return {
            'serviceActivated': selfv_cloud_market_cli_common.service_active,
            'serviceID': selfv_cloud_market_cli_common.service_id,
            'service': selfv_cloud_market_cli_common.service,
            'serviceStatuses[]': self.status,
            'userServiceStartFrom': self.start_from,
            'userServiceEndAt': self.end_at,
            'current': self.current,
            'pageSize': self.page_size
        }


class UserService:

    def __init__(self, net_type: str, password: str, nonce=0):
        self.cli, _ = NewServerWrapper(net_type, password, nonce)

    def get_user_service_page(self, current=1, page_size=10, statuses=None):
        '''
        Query user service via order info
        '''
        orderRoute = API_VERSION + '/order'

        activated = True
        if type(statuses) is not list:
            activated = statuses in ['ServicePending', 'ServiceRunning']
        opts = {
            'current': current,
            'pageSize': page_size,
            'serviceActivated': activated,
            'serviceStatuses[]': statuses if type(statuses) is list else [statuses]
        }

        opts = {k: v for k, v in opts.items() if v is not None}
        resp = self.cli.get_request(orderRoute, url_param=opts)
        ServiceCommon.validate_response(resp)
        return resp

    def get_user_service_info(self, provider_host: str, user_service_id: str):
        route = API_VERSION + '/userService/' + user_service_id
        tmp = self.cli.node_host
        self.cli.node_host = provider_host
        resp = self.cli.get_request(route, needAuth=True)
        ServiceCommon.validate_response(resp)
        self.cli.node_host = tmp
        return resp

    def access_user_api_get(self, user_service_id: str, api_type: str, api_func: str):
        route = API_VERSION + f'/service/userAPI/get/{api_type}/{api_func}/{user_service_id}'
        resp = self.cli.get_request(route, needAuth=True)
        return resp

    def access_user_api_post(self, user_service_id: str, api_type: str, api_func: str):
        pass

    def query_user_service(self, param: UserServiceQueryParam):
        route = API_VERSION + '/order'
        resp = self.cli.get_request(route, url_param=param.as_dict())
        ServiceCommon.validate_response(resp)
        return resp

    def _get_service_provider_api(self, provider_id: str):
        route = API_VERSION + '/service/provider/' + provider_id
        resp = self.cli.get_request(route)

        if isinstance(resp, dict):
            result = {
                'provider': resp.get('name', ''),
                'apiHost': resp.get('apiHost', '')
            }
        else:
            result = {
                'provider': '',
                'apiHost': ''
            }
        return result

    def _get_order_distinct_list(self, distinct_field: str, order_statuses: list):
        '''
        return: a list of distinct_field value
        '''
        route = API_VERSION + '/order/distinct'
        opts = {
            'distinctField': distinct_field,
            'statuses': order_statuses if type(order_statuses) is list else [order_statuses]
        }
        resp = self.cli.get_request(route, url_param=opts)
        ServiceCommon.validate_response(resp)
        return resp

    def check_image(self, image_info: str) -> bool:
        try:
            colon_index = image_info.rfind(":")
            image_name = image_info
            if colon_index < 0:
                return False
            image_name = image_info[:colon_index]
            tag = image_info[colon_index + 1:]
            route = f'/v1/repositories/{image_name}/tags/{tag}'
            temp = self.cli.node_host
            self.cli.node_host = "https://index.docker.io"
            resp = self.cli.get_request(route, raw_res=True)
            self.cli.node_host = temp
            return resp.status_code == 200
        except Exception as err:
            print(err)
        return False

    def check_ports(self, ports:list, region: str, provider_addr: str):
        temp = self.cli.node_host
        self.cli.node_host = provider_addr
        route = API_VERSION + '/k8s/checkports'
        data = {
            "region": region,
            "ports": ports
        }
        resp = self.cli.post_request(route, body_data=data)
        self.cli.node_host = temp
        ServiceCommon.validate_response(resp)
        return resp

    def start_usr_service(self, config:dict, route: str, secret: str) -> bool:
        temp = self.cli.node_host
        self.cli.node_host = ""
        headers = {
            "secret": secret
        }
        resp = self.cli.post_request(route, body_data=config, headers=headers, raw_res=True)
        self.cli.node_host = temp
        return resp.status_code == 200

    def service_status(self, provider_addr: str, secret: str):
        temp = self.cli.node_host
        self.cli.node_host = provider_addr
        route = API_VERSION + '/k8s/pod/status'
        headers = {
            "secret": secret
        }
        resp = self.cli.get_request(route, headers=headers)
        self.cli.node_host = temp
        ServiceCommon.validate_response(resp)
        return resp



import json

from v_cloud_market_cli_common.utils.server_api_wrapper import NewServerWrapper
from v_cloud_market_cli_common.config.server_config import API_VERSION
from .service_common import ServiceCommon

class ServiceTypeQueryParam:
    provider = ''
    category = ''
    name = ''
    current = 1
    page_size = 10

    def as_dict(self) -> dict:
        return {
            'provider': self.provider,
            'category': self.category,
            'name': self.name,
            'current': self.current,
            'pageSize': self.page_size
        }

class MarketService:

    def __init__(self, net_type, password, nonce=0):
        self.nonce = nonce
        self.cli, self.account = NewServerWrapper(net_type, password, nonce)
        self.baseRoute = API_VERSION + '/service'

    def get_service_info_page(self, opt, current=1, page_size=10):
        route = self.baseRoute + '/type'
        opts = {
            'current': current,
            'pageSize': page_size,
            'category': opt['category']
        }
        opts.update(opt)
        opts = {k: v for k, v in opts.items() if v}
        resp = self.cli.get_request(route, url_param=opts, needAuth=False)
        ServiceCommon.validate_response(resp)
        return resp

    def get_category_info_page(self, opt, current=1, page_size=10):
        route = self.baseRoute + '/category'
        opts = {
            'current': current,
            'pageSize': page_size
        }
        opts.update(opt)
        opts = {k: v for k, v in opts.items() if v is not None}
        resp = self.cli.get_request(route, url_param=opts, needAuth=False)
        ServiceCommon.validate_response(resp)
        return resp

    def get_provider_info_page(self, opt, current=1, page_size=10):
        route = self.baseRoute + '/provider'
        opts = {
            'current': current,
            'pageSize': page_size
        }
        opts.update(opt)
        opts = {k: v for k, v in opts.items() if v is not None}
        resp = self.cli.get_request(route, url_param=opts, needAuth=False)
        ServiceCommon.validate_response(resp)
        return resp

    def get_provider_service_type(self, opt, current=1, page_size=10):
        route = self.baseRoute + "/type"
        opts = {
            'current': current,
            'pageSize': page_size
        }
        opts.update(opt)
        opts = {k: v for k, v in opts.items() if v is not None}
        resp = self.cli.get_request(route, url_param=opts, needAuth=False)
        ServiceCommon.validate_response(resp)
        return resp

    def get_service_info(self, service_id):
        route = self.baseRoute + f'/type/{service_id}'
        resp = self.cli.get_request(route)
        ServiceCommon.validate_response(resp)
        return resp

    def get_service_category(self, category_id):
        route = self.baseRoute + f'/category/{category_id}'
        resp = self.cli.get_request(route)
        ServiceCommon.validate_response(resp)
        return resp

    def get_service_provider(self, provider):
        route = self.baseRoute + f'/provider/{provider}'
        resp = self.cli.get_request(route)
        ServiceCommon.validate_response(resp)
        return resp

    def query_service_info(self, params: ServiceTypeQueryParam):
        route = self.baseRoute + f'/type'
        resp = self.cli.get_request(route, params.as_dict())
        ServiceCommon.validate_response(resp)
        return resp

    def make_order(self, service_id: str, amt: float, option_payload: dict, time_period: int, expired_date=''):
        '''
        return
        id - order id
        recipient - order payment recipient
        amount: order amount
        '''
        order_payload = {'serviceID': service_id}
        order_payload.update({'serviceOptions': option_payload})
        order_payload.update({'duration': time_period})
        order_payload.update({'amount': amt})
        if expired_date:
            order_payload.update({'expiredDate': expired_date})
        orderRoute = API_VERSION + '/order/add'
        resp = self.cli.post_request(orderRoute, body_data=order_payload, needAuth=True)
        ServiceCommon.validate_response(resp)
        return resp

    def get_provider_host(self, provider: str) -> str:
        route = self.baseRoute + '/provider/' + provider

        # get provider host
        provider_cache_file = 'provider_cache.json'
        with open(provider_cache_file, 'w+') as cache:
            try:
                hold = json.load(cache)
            except json.JSONDecodeError:
                hold = {}
            if provider in hold:
                provider_host = hold[provider]
                return provider_host
            else:
                resp = self.cli.get_request(route)
                ServiceCommon.validate_response(resp)
                hold[provider] = resp['apiHost']
                json.dump(hold, cache)
                return hold[provider]

    def enough_balance(self, payAmount):
        info = self.account.balance_detail()
        if info is None or info['available'] < payAmount:
            return False
        return True

    def find_price_set(self, durations, options):
        for duration in durations:
            found = True
            for opt in duration['chargingOptions'].values():
                found = opt in options.values() and found
            if found:
                return duration



# PLATFORM_HOST = 'http://127.0.0.1:3002'
PLATFORM_HOST = 'https://o5mdw8b6mh.execute-api.us-east-1.amazonaws.com/test'
API_VERSION = '/api/v1'
NET = "T"

######## TESTING #################
MOCK_PAYMENT_SERVER = "http://127.0.0.1:3000"



import os
import urllib
import requests
import base64
import time
import json
import base58
import axolotl_curve25519 as curve
from urllib.parse import urlparse
from hashlib import sha256

from .vsyschain.crypto import str2bytes
from .wallet_cipher import WalletCipher
from v_cloud_market_cli_common.service.wallet_service import WalletService
from v_cloud_market_cli_common.config.server_config import PLATFORM_HOST

class ServerWrapper(object):

    def __init__(self, nodeHost, privKey=b'', pubKey=b'', address=''):
        self.node_host = nodeHost
        self.priv_key = privKey
        self.pub_key = pubKey
        self.user_agent = "market command line v1.0"
        self.address = address

    def get_request(self, api, url_param={}, body_data={}, headers={}, needAuth=False, raw_res=False):
        if url_param:
            url_param = {k: url_param[k] for k in url_param if url_param[k]}
            api += '?' + urllib.parse.urlencode(url_param, doseq=True)
        return self._form_request("GET", api, body_data, extra_headers=headers, needAuth=needAuth, raw_res=raw_res)

    def post_request(self, api, url_param={}, body_data={}, headers={}, needAuth=False, raw=False, raw_res=False):
        if url_param:
            url_param = {k: url_param[k] for k in url_param if url_param[k]}
            api += '?' + urllib.parse.urlencode(url_param, doseq=True)
        return self._form_request("POST", api, body_data, needAuth, extra_headers=headers, raw=raw, raw_res=raw_res)

    def _form_request(self, method, api, body_data, needAuth, extra_headers={}, raw=False, raw_res=False):
        curTime = int(time.time())
        headers = {
            "x-sdk-time": str(curTime),
            "User-Agent": self.user_agent,
            "address": self.address
        }
        if needAuth:
            msg = self._form_string_to_sign(
                curTime,
                self._form_canonical_request(method, api, body_data))

            sig = self.sign(msg)
            b58Pubkey = base58.b58encode(self.pub_key).decode('utf-8')
            headers['public-key'] = b58Pubkey
            headers['signature'] = sig
        headers = {**headers, **extra_headers}
        resp = None
        try:
            if method == "GET" and body_data:
                resp = requests.get(self.node_host + api, headers=headers, json=body_data)
            elif method == "GET" and not body_data:
                resp = requests.get(self.node_host + api, headers=headers)
            elif method == "POST":
                if raw:
                    resp = requests.post(self.node_host + api, headers=headers, data=body_data, timeout=30)
                else:
                    resp = requests.post(self.node_host + api, headers=headers, json=body_data)
        except requests.exceptions.Timeout as e:
            print(e)
        except json.decoder.JSONDecodeError:
            resp = {'message': 'JSONDecodeError: the response data is not json'}
        except Exception as err:
            print(err)
            resp = {"message": 'unknown error'}
        if raw_res or "message" in resp:
            return resp
        return resp.json()

    def _form_canonical_request(self, method, api, body_data={}):
        """
        canonical_request_string = \
            http_request_method + "\n" + \
            canonical_query_string + "\n" + \
            canonical_headers + "\n" + \
            SginedHeaders + "\n" + \
            HexEncode(Hash(RequestPayload))
        """
        m = sha256()
        u = urlparse(self.node_host + api)
        path = u.path
        if not path.startswith("/api/v1"):
            path = path[path.find('/api/v1'):]
        reqString = method + '\n' + \
                    path + '\n' + \
            '\n'.join(u.query.split('&')) + '\n' \
            'User-Agent:' + self.user_agent + '\n' + \
            'address:' + self.address + '\n'
        if body_data is not None:
            body_data = sha256(json.dumps(body_data).encode('utf-8')).hexdigest()
            reqString += body_data

        m.update(reqString.encode('utf-8'))
        return str(m.hexdigest())

    def _form_string_to_sign(self, timestamp, hashed):
        return sha256(("HMAC-SHA256" + str(timestamp) + hashed).encode('utf-8')).digest()

    def sign(self, inputByte):
        randm64 = os.urandom(64)
        sig = curve.calculateSignature(randm64, self.priv_key, inputByte)
        b64sig = base64.b64encode(sig).decode('utf-8')
        return b64sig


def NewServerWrapper(net_type, password, nonce, server_host=PLATFORM_HOST):
    walletService = WalletService(None, net_type, password)
    if len(walletService.accounts) - 1 < nonce:
        print(f"Info: wallet only contains address of nonce {len(walletService.accounts) - 1}, use nonce 0")
        nonce = 0
    account = walletService.accounts[nonce]
    pubKey = base58.b58decode(str2bytes(account.publicKey))
    privKey = base58.b58decode(str2bytes(account.privateKey))
    address = WalletCipher.generate_address(pubKey, net_type).decode('utf-8')
    return ServerWrapper(PLATFORM_HOST, privKey, pubKey, address), account



from rich.console import Console
from pyfiglet import Figlet

class MainInterface:

    def __init__(self):
        pass

    @staticmethod
    def display_title(console: Console, title: str):
        align = 'left'
        f = Figlet(font='big')
        console.clear()
        splitted = f.renderText(title).split("\n")
        length = len(splitted[0])
        titleBox = ''

        for i in range(len(splitted)-1):
            titleBox += '||    ' + splitted[i] + '    ||\n'

        console.print('=' * (length + 12), justify=align)
        console.print(titleBox + '=' * (length + 12), justify=align)

    @staticmethod
    def display_service_choice(console: Console, title: str, choices: [str], isSub=False):
        MainInterface.display_title(console, title)
        align = 'left'
        if isSub:
            console.print('[red][B][/] Press B to UpperLevel', justify=align)

        style = 'bold blue'
        index = 1
        for choice in choices:
            numPrefix = f'[{index}]' + ' ' * 4
            console.print(numPrefix+choice, style=style, justify=align)
            index += 1
        console.line(1)
        return console.input('[dark_sea_green4]Please Select Your Choice[/]: ')









# rfcs/0007-wallet-interaction-specification-4
from rich.console import Console
import json
import pyqrcode


class QRCodeDisplay:

    def __init__(self):
        pass

    def show_export_seed(self, seed: str):
        payload = {
            'protocol': 'v.systems',
            'api': 1,
            'opc': 'seed',
            'seed': seed
        }
        self._display_qr_code(payload)

    def show_account_of_wallet(self, address: str, public_key: str = '', amt: float = 0, invoice: str = ''):
        '''
        api: if it has invoice, api should be 2. Otherwise, it's 1
        amount: should not be specified when monitoring cold wallet account in hot wallet
        publicKey: is required for monitoring cold wallet but optional for receiving coin
        '''
        payload = {
            'protocol': 'v.systems',
            'opc': 'account',
            'address': address
        }

        if invoice:
            payload['invoice'] = invoice
            payload['api'] = 2
        else:
            payload['api'] = 1

        if amt and amt > 0:
            payload['amount'] = int(amt * 10**8)

        if public_key:
            payload['publicKey'] = public_key
        self._display_qr_code(payload)

    def _display_qr_code(self, payload: dict):
        text = pyqrcode.create(json.dumps(payload), error='L')
        print(text.terminal(quiet_zone=1))


from datetime import datetime
from rich.console import Console
from rich.table import Table
from rich.measure import Measurement
from rich.panel import Panel
from rich import box

from v_cloud_market_cli_common.service_display.display_common import utc_to_local


class OrderServiceDisplay:

    def __init__(self, console: Console):
        self.console = console

    def display_order_page(self, title: str, page_result: dict) -> int:
        self.console.clear()
        table = Table(show_header=True, header_style='magenta')
        if len(page_result["list"]) > 0:
            table.title = '[bold magenta]' + title + f' --- Address: {page_result["list"][0]["address"]}'
        else:
            table.title = '[bold magenta]' + title + ' --- Address: there is no pending orders'
        table.box = box.SIMPLE_HEAD
        table.add_column('Order ID', justify='center')
        table.add_column('Order Status', justify='center')
        table.add_column('Creation Time', justify='center')
        table.add_column('Service Type', justify='center')
        table.add_column('Recipient Address', justify='center')
        table.add_column('Duration (hour)', '[u]432,000,000', justify='right')
        table.add_column('Amount (VSYS)', '[u]412,000,000', justify='right')
        table.add_column('Amount Paid (VSYS)', '[u]412,000,000', justify='right')
        table.add_column('Service Enabled', justify='center')
        for row in self.form_order_rows(page_result['list']):
            table.add_row(*row)
        self.console.print(table, justify='center')
        return Measurement.get(self.console, table).maximum

    def display_order_info(self, order_info):
        timeStr = utc_to_local(datetime.utcfromtimestamp(int(order_info['createdAt']))).strftime('%Y-%m-%d %H:%M:%S')
        optionMsg = ''
        for k in order_info['serviceOptions']:
            optionMsg += ' ' * 21 + f'[bright_green]' + k + ':[/] ' + order_info['serviceOptions'][k] + '\n'

        msg = '[bold magenta]Address:[/] ' + ' ' * 12 + order_info['address'] + '\n' + \
              '[bold magenta]Order ID:[/] ' + ' ' * 11 + order_info['id'] + '\n' + \
              '[bold magenta]Service:[/] ' + ' ' * 12 + order_info['service'] + '\n' + \
              '[bold magenta]Service ID:[/] ' + ' ' * 9 + order_info['serviceID'] + '\n' + \
              '[bold magenta]Recipient:[/] ' + ' ' * 10 + order_info['recipient'] + '\n' + \
              '[bold magenta]Creation Time:[/] ' + ' ' * 6 + timeStr + '\n' + \
              '[bold magenta]Duration (hour):[/] ' + ' ' * 4 + str(order_info['duration']) + '\n' + \
              '[bold magenta]Amount (VSYS):[/] ' + ' ' * 6 + str(order_info['amount']) + '\n' + \
              '[bold magenta]Amount Paid (VSYS):[/] ' + ' ' + str(order_info['amountPaid']) + '\n' + \
              '[bold magenta]Status:[/] ' + ' ' * 13 + str(order_info['status']) + '\n' + \
              '[bold magenta]Service Enabled:[/]' + ' ' * 5 + str(order_info['serviceActivated']) + '\n' + \
              '[bold magenta]Service Options:[/] \n%s' % optionMsg
        self.console.print(Panel.fit(msg))
        self.console.input('Press ENTER to continue...')

    def form_order_rows(self, orderList: [dict]) -> [[str]]:
        rows = []
        for order in orderList:
            row = [order['id'], order['status']]
            time_dt = datetime.utcfromtimestamp(int(order['createdAt']))
            timeStr = utc_to_local(time_dt).strftime('%Y-%m-%d %H:%M:%S')
            row.extend([
                timeStr,
                order['service'],
                order['recipient'],
                str(order['duration']),
                str(order['amount']),
                str(order['amountPaid']),
                'true' if order['serviceActivated'] else 'false'
            ])
            rows.append(row)
        return rows



VSYS = 100000000
DEFAULT_TX_FEE = int(0.1 * VSYS)
DEFAULT_FEE_SCALE = 100

DEFAULT_PAYMENT_FEE = DEFAULT_TX_FEE
DEFAULT_LEASE_FEE = DEFAULT_TX_FEE
DEFAULT_CANCEL_LEASE_FEE = DEFAULT_TX_FEE
CHECK_FEE_SCALE = True

MAX_ATTACHMENT_SIZE = 140
MAX_NONCE = 4294967295
MAX_TX_HISTORY_LIMIT = 10000

DEFAULT_CHAIN = 'mainnet'
DEFAULT_CHAIN_ID = 'M'

TESTNET_CHAIN = 'testnet'
TESTNET_CHAIN_ID = 'T'

DEFAULT_TESTNET_NODE = 'https://test.v.systems/api'
DEFAULT_TESTNET_API_KEY = ''

DEFAULT_NODE = 'http://54.147.255.148:9922'
DEFAULT_NODE_API_KEY= ''

ADDRESS_VERSION = 5
ADDRESS_CHECKSUM_LENGTH = 4
ADDRESS_HASH_LENGTH = 20
ADDRESS_LENGTH = 1 + 1 + ADDRESS_CHECKSUM_LENGTH + ADDRESS_HASH_LENGTH

PAYMENT_TX_TYPE = 2
LEASE_TX_TYPE = 3
LEASE_CANCEL_TX_TYPE = 4

THROW_EXCEPTION_ON_ERROR = True



'''
last editted: 2021-04-23
version: 0.1.0
'''
from main_logic import start
import logging
from configparser import ConfigParser

def main():
    # config = ConfigParser()
    # config.read("./market_place_cli/config/log.config", encoding="UTF-8")
    # logging.basicConfig(filename=f'{config["log"]["log_dir"]}{config["log"]["filename"]}',
    #                     level=logging.DEBUG,
    #                     format=config["log"]["LOG_FORMAT"],
    #                     datefmt=config["log"]["DATE_FORMAT"],
    #                     )
    start()

if __name__ == '__main__':
    main()



from v_cloud_market_cli_common.utils.server_api_wrapper import NewServerWrapper
from v_cloud_market_cli_common.config.server_config import API_VERSION, MOCK_PAYMENT_SERVER
from .service_common import ServiceCommon

class OrderQueryParam:
    id = ''
    start_from = 0
    end_at = 0
    service_id = ''
    service = ''
    recipient = ''
    statuses = []
    service_activated = False
    current = 1
    page_size = 10

    def as_dict(self) -> dict:
        return {
            'id': self.id,
            'startFrom': self.start_from,
            'serviceID': selfv_cloud_market_cli_common.service_id,
            'service': selfv_cloud_market_cli_common.service,
            'recipient': self.recipient,
            'statuses[]': self.statuses,
            'serviceActivated': selfv_cloud_market_cli_common.service_activated,
            'current': self.current,
            'pageSize': self.page_size
        }


class OrderService:

    def __init__(self, net_type: str, password: str, nonce=0):
        self.cli, self.account = NewServerWrapper(net_type, password, nonce)

    def get_order_info_page(self, current=1, page_size=10, status='OrderPending'):
        orderRoute = API_VERSION + '/order'
        opts = {
            'current': current,
            'pageSize': page_size,
            'statuses[]': [status]
        }

        if status in ['OrderPaid', 'OrderFiled']:
            opts['serviceActivated'] = True

        opts = {k: v for k, v in opts.items() if v is not None}
        resp = self.cli.get_request(orderRoute, url_param=opts)
        ServiceCommon.validate_response(resp)
        return resp

    def query_order_info(self, params: OrderQueryParam):
        orderRoute = API_VERSION + '/order'
        resp = self.cli.get_request(orderRoute, url_param=params.as_dict())
        ServiceCommon.validate_response(resp)
        return resp

    def get_order_info(self, order_id: str):
        """
        return:
        {
            "createdAt"
            "updatedAt"
            "duration"
            "serviceExpiredDate"
            "amount"
            "amountPaid"
            "serviceID"
            "service"
            "address"
            "recipient"
            "id"
            "serviceOptions"
            "status"
            "publicKey"
        }
        """
        route = API_VERSION + '/order/id/' + order_id
        resp = self.cli.get_request(route, needAuth=True)
        ServiceCommon.validate_response(resp)
        return resp
    
    def refund_order(self, provider_host: str, order_id: str):
        route = API_VERSION + '/order/userRefund/' + order_id
        tmp = self.cli.node_host
        self.cli.node_host = provider_host
        resp = self.cli.post_request(route, needAuth=True)
        ServiceCommon.validate_response(resp)
        self.cli.node_host = tmp
        return resp

    def mock_order_payment(self, order_id: str, provider_addr: str, pub_key: str, amt: int):
        route = API_VERSION + '/order/mock'
        order_mock_payload = {
            'orderId': order_id,
            'providerAddr': provider_addr,
            'pubKey': pub_key,
            'amt': amt * 10**8
        }
        node_host = self.cli.node_host
        self.cli.node_host = MOCK_PAYMENT_SERVER
        resp = self.cli.post_request(route, body_data=order_mock_payload)
        self.cli.node_host = node_host
        ServiceCommon.validate_response(resp)
        return resp



from threading import Lock
import requests
import time
import json

class ServerStream(object):
    def __init__(self, max_retries=10, wait_seconds=5):
        self.is_open = False
        self.max_retries = max_retries
        self.wait_seconds = wait_seconds
        self.lock = Lock()
        self.response = None
        self.iterator = None

    def open(self, url: str, headers: dict, payload: dict, func: callable):
        self.lock.acquire()
        if self.is_open:
            self.lock.release()
            return
        self.is_open = True
        self.retry_time = self.max_retries
        self.lock.release()
        # print new data if stream reconnected
        total_line_num = 0
        while self.retry_time > 0:
            try:
                line_num = 0
                self.response = requests.get(url,
                                    headers=headers,
                                    params=payload,
                                    stream=True,
                                    timeout=(3, 3))
                if self.response.status_code != 200:
                    raise Exception(
                        "Cannot get stream (HTTP {}): {}".format(
                            self.response.status_code, self.response.text
                        )
                    )
                self.iterator = self.response.iter_lines()
                while self.is_open:
                    self.lock.acquire()
                    data = next(self.iterator)
                    self.lock.release()
                    # filter out keep-alive new lines
                    if data:
                        self.retry_time = self.max_retries
                        line_num += 1
                        if line_num > total_line_num:
                            func(data)
                        total_line_num = max(total_line_num, line_num)
            except requests.exceptions.RequestException:
                pass
            except requests.exceptions.ChunkedEncodingError:
                time.sleep(self.wait_seconds)
            except Exception as err:
                if len(str(err)) != 0:
                    self.retry_time -= 1
                time.sleep(self.wait_seconds)
            finally:
                if self.lock.locked():
                    self.lock.release()
        else:
            if self.is_open:
                print("User cli tool failed to reconnect to server!")
            self.close()

    def close(self):
        self.lock.acquire()
        self.is_open = False
        self.retry_time = 0
        if self.iterator:
            self.iterator.close()
        if self.response:
            self.response.close()
        self.lock.release()



from base58 import b58encode
from v_cloud_market_cli_common.service_display.wallet_service_display import WalletServiceDisplay

from v_cloud_market_cli_common.utils.wallet_storage import WalletStorage, WalletData
from v_cloud_market_cli_common.utils.vsyschain.account import *
from v_cloud_market_cli_common.utils.vsyschain import testnet_chain
from v_cloud_market_cli_common.config.wallet_config import (
    WALLET_FILENAME,
    AGENT_STRING,
    ADDRESS_CSV_FILENAME
)

NO_DISPLAY = 0
DISPLAY_BALANCE = 1
DISPLAY_BALANCE_DETAIL = 2


class WalletService:

    def __init__(self, wsd: WalletServiceDisplay, net: str, password: str, walletFile=None, load_wallet=True, show_err=True):
        self.display = wsd
        self.accounts = []
        self.net = net
        self.password = password
        self.cipher = None if not password else WalletCipher(password)
        self.show_err = show_err
        if walletFile:
            self.walletFile = walletFile
        else:
            self.walletFile = WALLET_FILENAME
        if load_wallet:
            self.wallet_data = self.load_wallet_file()

    def load_wallet_file(self):
        wallet_storage = WalletStorage(self.walletFile)
        try:
            wallet_data = wallet_storage.load(self.cipher, show_err=self.show_err)
            if not wallet_data or len(wallet_data) == 0:
                return None
        except Exception as e:
            return None
        self._load_accounts(seed=wallet_data.seed, nonce=wallet_data.nonce)
        return wallet_data

    def save_wallet_file(self, walletData):
        wallet_storage = WalletStorage('./' + WALLET_FILENAME)
        wallet_storage.save(walletData, self.cipher)

    def set_wallet_cipher(self, new_password: str):
        self.cipher = None if not new_password else WalletCipher(new_password)

    def decrypt_wallet_file(self):
        data = self.load_wallet_file()
        if data:
            self.save_wallet_file(data)

    def show_address(self, toCSV, balanceFlag):
        if not self.wallet_data:
            self.display.console.print('[orange]No seed information or wallet data file.')
            return
        seed = self.wallet_data.seed
        accInfoList = []
        for i in range(0, len(self.wallet_data.accountSeeds)):
            accInfo = WalletCipher.generate_account_info(seed, self.net, i)
            if balanceFlag == DISPLAY_BALANCE:
                accInfo['balance'] = self._load_account(seed, i).balance()
            elif balanceFlag == DISPLAY_BALANCE_DETAIL:
                accInfo['balanceDetail'] = self._load_account(seed, i).balance_detail()
            accInfoList.append(accInfo)
            if toCSV:
                accInfo['seed'] = self.wallet_data.seed
                self._save_to_csv(accInfo)
        self.display.show_address(accInfoList)

    def fetch_wallet_info(self, index: int, field: str):
        if not self.wallet_data:
            self.display.console.print('[orange]No seed information or wallet data file.')
            return
        account = self.accounts[index]

        pub = base58.b58decode(str2bytes(account.publicKey))
        # priv = base58.b58decode(str2bytes(account.privateKey))
        address = WalletCipher.generate_address(pub, self.net).decode('utf-8')
        if field == 'priv':
            return account.privateKey
        elif field == 'pub':
            return account.publicKey
        elif field == 'add':
            return address

    def recover_wallet(self, seed, count, toCSV, balanceFlag=DISPLAY_BALANCE_DETAIL):
        accSeeds = set()
        accInfoList = []
        for i in range(0, count):
            accInfo = WalletCipher.generate_account_info(seed, self.net, i)
            accInfo['seed'] = seed
            if balanceFlag == DISPLAY_BALANCE:
                accInfo['balance'] = self._load_account(seed, i).balance()
            elif balanceFlag == DISPLAY_BALANCE_DETAIL:
                accInfo['balanceDetail'] = self._load_account(seed, i).balance_detail()
            accInfoList.append(accInfo)
            accSeedDisplay = b58encode(accInfo['accSeed']).decode('utf-8')
            accSeeds.add(accSeedDisplay)
            if toCSV:
                self._save_to_csv(accInfo)
            else:
                self._clean_csv_file()
        data = WalletData(seed, list(accSeeds), count, AGENT_STRING)
        self.save_wallet_file(data)
        self.display.show_address(accInfoList)

    def seed_generate(self, toCSV, balanceFlag, count=1):
        seed = WalletCipher.generate_phrase()
        self._protect_local_wallet()
        self.display.show_seed(seed)
        self.recover_wallet(seed=seed, count=count, toCSV=toCSV, balanceFlag=balanceFlag)

    def address_generate(self, count, toCSV, toAppend, balanceFlag):
        if not self.wallet_data:
            self.seed_generate(count, toCSV)
            self.load_wallet_file()

        n = self.wallet_data.nonce - 1
        seed = self.wallet_data.seed
        oldAccSeeds = self.wallet_data.accountSeeds
        newAcc = set()
        accInfoList = []
        for i in range(n, n + count):
            accInfo = WalletCipher.generate_account_info(seed, self.net, i)
            accInfo['seed'] = seed
            accSeedDisplay = b58encode(accInfo['accSeed']).decode('utf-8')
            newAcc.add(accSeedDisplay)
            accInfoList.append(accInfo)
            if balanceFlag == DISPLAY_BALANCE:
                accInfo['balance'] = self._load_account(seed, i).balance()
            elif balanceFlag == DISPLAY_BALANCE_DETAIL:
                accInfo['balanceDetail'] = self._load_account(seed, i).balance_detail()
            if toCSV:
                self._save_to_csv(accInfo)
        if toAppend:
            oldAccSeeds += list(newAcc)
            data = WalletData(seed, oldAccSeeds, count + n, AGENT_STRING)
            self.save_wallet_file(data)
        self.display.show_address(accInfoList)

    def account_pay(self, accountNonce, recipient, amount, attachment=''):
        account = self.accounts[accountNonce]
        resp = account.send_payment(recipient, amount * 10**8, attachment=attachment)
        self.display.show_account_pay(recipient, resp['amount'] / 10 ** 8)

    def _protect_local_wallet(self):
        if os.path.exists(WALLET_FILENAME):
            input('> press ENTER to continue.')

    def _save_to_csv(self, jsonData):
        with open(ADDRESS_CSV_FILENAME, 'w+') as file:
            file_str = str(jsonData['nonce']) + ',' + \
            jsonData['addr'] + ',' + \
            b58encode(jsonData['pub']).decode('utf-8') + ',' + \
            b58encode(jsonData['priv']).decode('utf-8') + ',' + \
            b58encode(jsonData['accSeed']).decode('utf-8') + ',' + \
            jsonData['seed']
            if self.cipher:
                file.write(self.cipher.encrypt(file_str))
            else:
                file.write(file_str)

    def _clean_csv_file(self):
        with open(ADDRESS_CSV_FILENAME, 'w+') as file:
            file.write('')

    def _load_accounts(self, seed, nonce):
        for i in range(0, nonce):
            self.accounts.append(self._load_account(seed, i))

    def _load_account(self, seed, nonce):
        if self.net == 'M':
            return Account(chain=mainnet_chain(), seed=seed, nonce=nonce)
        else:
            return Account(chain=testnet_chain(), seed=seed, nonce=nonce)



import datetime
import json
import threading

from rich.console import Console
from rich.panel import Panel
from rich.progress import Progress


from market_place_cli.v_cloud_market_cli_common.service.wallet_service import WalletService, WalletCipher
from market_place_cli.v_cloud_market_cli_common.service_display.main_interface import MainInterface
from market_place_cli.v_cloud_market_cli_common.service_display.user_service_display import UserServiceDisplay
from market_place_cli.v_cloud_market_cli_common.service_display.market_service_display import MarketServiceDisplay
from market_place_cli.v_cloud_market_cli_common.service.user_service import UserService
from market_place_cli.v_cloud_market_cli_common.service.market_service import MarketService
from market_place_cli.service_interface_request.wallet_service_request import WalletRequest
from market_place_cli.service_interface_request.user_service_request import UserServiceRequest
from market_place_cli.service_interface_request.common_request import get_table_choice
from market_place_cli.service_interface_logic.common import get_net, wallet_has_password
from market_place_cli.v_cloud_market_cli_common.utils.message_decipher import decrypt_message
from market_place_cli.v_cloud_market_cli_common.utils.server_stream import ServerStream
from market_place_cli.utils.string_utils import get_container_memory


class UserServiceLogic:

    def __init__(self):
        self.title = 'UserService'
        self.console = None
        self.ur = None  # User Request
        self.ud = None  # User Display
        self.net_type = 'M'
        self.main_functions = ['Show Running User Service', 'Show Usable User Service', 'Show Past User Service', 'Show Abort User Service']
        self.interface_index = 0 
        self.stream = ServerStream()
        self.password = None
        self.wr = None


    @property
    def Name(self):
        return self.title

    def StartLogic(self, console: Console, isTestnet: bool):
        self.console = console
        self.wr = WalletRequest(self.console)
        self.net_type = get_net(isTestnet)
        self.ur = UserServiceRequest(self.console)
        self.ud = UserServiceDisplay(self.console)
        console.clear()
        while True:
            choice = MainInterface.display_service_choice(console, self.title, self.main_functions, True)
            if choice == '1':
                self.interface_index = 1
                self.show_current_user_service_logic()
            elif choice == '2':
                self.interface_index = 2
                self.show_usable_user_service_logic()
            elif choice == '3':
                self.interface_index = 3
                self.show_past_user_service_logic()
            elif choice == '4':
                self.interface_index = 4
                self.show_abort_user_service_logic()
            elif choice.lower() == 'b':
                break
            self.interface_index = 0

    def show_current_user_service_logic(self):
        self.show_user_service_page("ServiceRunning")

    def show_usable_user_service_logic(self):
        self.show_user_service_page("ServiceUsable")

    def show_past_user_service_logic(self):
        self.show_user_service_page("ServiceDone")

    def show_abort_user_service_logic(self):
        self.show_user_service_page("ServiceAbort")

    def access_provider_api_logic(self, us: UserService, user_service_info: dict, index: int):
        try:
            # get decryption private key
            private_key = WalletService(None, self.net_type, self.password).fetch_wallet_info(index, "priv")

            # get provider host
            ms = MarketService(self.net_type, self.password, index)
            provider_host = ms.get_provider_host(user_service_info["provider"])

            info = us.get_user_service_info(provider_host, user_service_info["id"])
            magic = info["magic"]
            cipher = decrypt_message(private_key, magic)
            p = Panel.fit(cipher)
            p.title = "Service Login Information"
            p.title_align = "center"
            self.console.print(p)
            self.console.input("Press ENTER to continue...")
        except Exception as e:
            print(e)
            self.console.input("[bright_red]Failed to get userservice info. Press ENTER to continue...")

    # To-Do: refactor function 
    def access_user_service_api_logic(self, us: UserService, user_service_info: dict, index: int):
        md = MarketServiceDisplay(self.console)
        ms = MarketService(self.net_type, self.password, index)
        service_info = ms.get_service_info(user_service_info['serviceID'])
        self.console.print(md.form_service_api(service_info['serviceAPI']))
        is_secret, api_func = self.ur.get_api_func(service_info['serviceAPI'])
        if is_secret:
            msg = us.access_user_api_get(user_service_id, 'secret', api_func)
        else:
            msg = us.access_user_api_get(user_service_id, 'normal', api_func)
        p = Panel.fit(msg)
        p.title = '[]User Service API Service Response'
        p.title_align = 'center'
        self.console.print(p)
        self.console.input("Press ENTER to continue...")

    def start_user_service_api_logic(self, us: UserService, order_info: dict, index: int):

        if order_info["serviceStatus"] != "ServicePending":
            self.console.print(f"Order service status is { order_info['status'] }, and cannot start service.")
            self.console.input("Press ENTER to continue...")
            return
        # get provider host
        full_user_service_info = self._get_us_info_with_secret(us, order_info, index)
        if full_user_service_info is None or "Secret" not in full_user_service_info:
            self.console.print(f"[bright_green]Order {full_user_service_info['id']} has invalid magic!")
            self.console.input("Press ENTER to continue...")
            return

        image_name = self.ur.get_image_info(us.check_image)
        # get container and host post persistStorage
        serviceOptions = full_user_service_info["serviceOptions"]
        port_configs = self.ur.get_ports(serviceOptions, full_user_service_info["provider_host"], us.check_ports)
        memory = get_container_memory(serviceOptions["resourceUnit"])
        container_config = {
            "name": "container-0",
            "imageName": image_name,
            "ports": port_configs,
            "resource": {
                "memory": memory
            }
        }
        if "persistStorage" in serviceOptions and serviceOptions["persistStorage"] == "yes":
            container_config["mountPath"] = self.ur.get_mount_path()
        cmd = self.console.input("Please input command(optional) : ")
        if len(cmd.replace(" ", "")) > 0:
            container_config["command"] = cmd
        args = self.console.input("Please input arguments to (optional) : ")
        if len(args) != 0:
            container_config["args"] = args
        service_config = {
            "name": order_info["id"],
            "containers": [container_config]
        }
        try:
            if not us.start_usr_service(service_config, full_user_service_info["Ip"], full_user_service_info["Secret"]):
                self.console.print(f"[bright_red]Failed to start service { order_info['id'] }")
            else:
                self.console.print(f"[bright_green]Order { order_info['id'] } user service has been deployed successfully!")
        except Exception as e:
            print(e)
            self.console.print(f"[bright_red]Failed to start service { order_info['id'] }")
        self.console.input("Press ENTER to continue...")

    def tail_service_log_logic(self, us: UserService, order_info: dict, index: int):
        user_service_info = self._get_us_info_with_secret(us, order_info, index)
        if user_service_info is None or "Secret" not in user_service_info:
            self.console.print(f"[bright_green]Order {order_info['id']} has invalid magic!")
            self.console.input("Press ENTER to continue...")
            return
        url = user_service_info["provider_host"] + "/api/v1/k8s/pod/logs"
        headers = {
            "Connection": "close",
            "secret": user_service_info["Secret"]
        }
        self.console.clear()
        self.console.print("[bright_green]Press ENTER to stop log...")
        log_thread = threading.Thread(target=self.stream.open, args=(url, headers, None, self.read_stream))
        log_thread.daemon = True
        log_thread.start()
        self.console.input("")
        # close stream
        self.stream.close()

    def monitor_services(self, us: UserService, orders: list, index: int):
        status_list = []
        with Progress() as progress:
            task = progress.add_task("[green]Querying data...", total=len(orders) * 2, start=False)
            progress.start_task(task)
            try:
                for order in orders:
                    if order["serviceStatus"] != "ServiceRunning":
                        continue
                    user_service = self._get_us_info_with_secret(us, order, index)
                    progress.update(task, advance=1)
                    if user_service is None:
                        continue
                    status_data = us.service_status(user_service["provider_host"], user_service["Secret"])
                    progress.update(task, advance=1)
                    if status_data is None:
                        continue
                    item = {
                        "id": order["id"],
                        "region": order["serviceOptions"]["region"]
                    }
                    if "podStatus" in status_data and "containerStatuses" in status_data["podStatus"]:
                        container_status = status_data["podStatus"]["containerStatuses"][0]
                        item["image"] = container_status["image"]
                        state = list(container_status["state"].keys())
                        item["state"] = state[0]
                        item["restartCount"] = container_status["restartCount"]
                        item["hostIP"] = status_data["podStatus"]["hostIP"]

                    if "podSpec" in status_data and "containers" in status_data["podSpec"]:
                        # only one container in each pod
                        container = status_data["podSpec"]["containers"][0]
                        if "ports" in container:
                            item["ports"] = container["ports"]
                    if "oomNotice" in status_data and "history" in status_data["oomNotice"]:
                        item['oomHistory'] = []
                        for timestamp in status_data["oomNotice"]["history"]:
                            local_time = datetime.datetime.fromtimestamp(timestamp)
                            item['oomHistory'].append(str(local_time))
                    status_list.append(item)
            except Exception as e:
                print(e)
                self.console.input("Press ENTER to continue...")
                return
        headers = [
            { "text": "Order ID", "value": "id"},
            { "text": "Region", "value": "region"},
            { "text": "Image", "value": "image"},
            { "text": "Container Status", "value": "state"},
            # { "text": "Restart Count", "value": "restartCount"},
            { "text": "Host IP", "value": "hostIP"},
            { "text": "Port Spec", "value": "ports", "justify": "left"},
            { "text": "Out of Memory History", "value": "oomHistory"},
        ]
        self.ud.display_service_status(headers, status_list)
        self.console.input("Press ENTER to continue...")

    def show_user_service_page(self, status: str):
        index = self.wr.get_payment_address()
        if wallet_has_password(self.net_type):
            self.password = self.wr.get_dec_password()

        us = UserService(self.net_type, self.password, index)
        cur = 1
        page_size = 10
        title, extra = self._construct_page_title(status)
        while True:
            try:
                display_result, has_next = self._construct_user_service_page(us, cur, status)
            except Exception as e:
                self.console.print(e)
                self.console.input("[bright_red]Failed to get userservice info. Press ENTER to continue...")
                break
            w = self.ud.show_user_service_page(title, display_result['list'])
            has_next = len(display_result['list']) >= page_size
            choice = get_table_choice(self.console, w, has_next, extra=extra)
            if choice == 'p' and cur > 1:
                cur -= 1
            elif choice == 'n' and has_next:
                cur += 1
            elif choice == 'e':
                break
            elif choice == 'd':
                user_service_id = self.ur.get_user_service_id()
                u, ok = self.validate_user_service(display_result, user_service_id)
                if ok:
                    self.ud.show_user_service_detail(u)
            elif status == 'ServiceUsable' and choice == 'u':
                user_service_id = self.ur.get_user_service_id()
                us_info, ok = self.validate_user_service(display_result, user_service_id)
                if ok:
                    self.access_provider_api_logic(us, us_info, index)
            elif status == 'ServiceUsable' and choice == 's':
                user_service_id = self.ur.get_user_service_id()
                us_info, ok = self.validate_user_service(display_result, user_service_id)
                if ok:
                    self.start_user_service_api_logic(us, us_info, index)
            elif choice == 'a':
                user_service_id = self.ur.get_user_service_id()
                us_info, ok = self.validate_user_service(display_result, user_service_id)
                if ok:
                    self.access_user_service_api_logic(us, us_info, index)
            elif self.interface_index == 1 and choice == 't':
                user_service_id = self.ur.get_user_service_id()
                us_info, ok = self.validate_user_service(display_result, user_service_id)
                if ok:
                    self.tail_service_log_logic(us, us_info, index)
            elif self.interface_index == 1 and choice == 'm':
                order_id = self.console.input('[bright_green]Please enter order id: ')
                temp_list = []
                for item in display_result["list"]:
                    if item["id"] == order_id:
                        temp_list.append(item)
                if len(temp_list) == 0:
                    temp_list = display_result["list"]
                self.monitor_services(us, temp_list, index)

    def validate_user_service(self, result: dict, user_service_id: str) -> (dict, bool):
        found = False
        for u in result['list']:
            if u['id'] == user_service_id:
                found = True
                return u, found
        if not found:
            self.console.input("[bright_red]User Service ID Not Found.[/] Press ENTER to continue...")
            return None, found

    def _construct_user_service_page(self, us: UserService, cur_page: int, status: str):
        if status == 'ServiceUsable':
            display_result = us.get_user_service_page(
                current=cur_page,
                page_size=10,
                statuses=['ServicePending', 'ServiceRunning'])
        else:
            display_result = us.get_user_service_page(
                current=cur_page,
                page_size=10,
                statuses=status)

        has_next = False if len(display_result['list']) < 10 else True
        return display_result, has_next

    def _construct_page_title(self, status: str):
        extra = {}
        title = 'User Service Information Table'
        if status == 'ServiceRunning':
            title = 'Running ' + title
            extra = {'d': '[D]User Service Detail', 'a': '[A]User Service API Access', 't': '[T]Tail Service Log', 'm': '[M]Monitor Service In Current Page'}
        elif status == 'ServiceUsable':
            title = 'Usable ' + title
            extra = {'d': '[D]User Service Detail', 'u': '[U]User Service Usage Info', 's': '[S]Start a service'}
        elif status == 'ServiceDone':
            title = 'Past ' + title
        return title, extra

    def _get_us_info_with_secret(self, us: UserService, order_info: dict, wallet_index: int) -> dict:
        # get decryption private key
        try:
            private_key = WalletService(None, self.net_type, self.password).fetch_wallet_info(wallet_index, "priv")
            ms = MarketService(self.net_type, self.password)
            provider_host = ms.get_provider_host(order_info['provider'])
            info = us.get_user_service_info(provider_host, order_info["id"])
            info["provider_host"] = provider_host
            plain_txt_magic = decrypt_message(private_key, info["magic"])
            magic_dict = json.loads(plain_txt_magic)
            return { **magic_dict, **info }
        except Exception as err:
            print(err)
            return None

    def read_stream(self, data):
        line = data.decode("utf-8")
        if line.startswith("data:"):
            line = line[5:]
        if line != "\"logger - Client disconnected.\"":
            self.console.print(line)



def get_unit_num(resource_unit:str) -> str:
    index = resource_unit.find("-")
    return int(resource_unit[:index])

def get_container_memory(resource_unit:str) -> str:
    unit = get_unit_num(resource_unit)
    return str(unit) + "Gi"



import math
from rich.console import Console

from market_place_cli.v_cloud_market_cli_common.service.wallet_service import WalletService
from market_place_cli.v_cloud_market_cli_common.service_display.market_service_display import MarketServiceDisplay
from market_place_cli.v_cloud_market_cli_common.service_display.qr_code_display import QRCodeDisplay
from market_place_cli.v_cloud_market_cli_common.service_display.main_interface import MainInterface
from market_place_cli.v_cloud_market_cli_common.service.market_service import MarketService
from market_place_cli.service_interface_request.wallet_service_request import WalletRequest
from market_place_cli.service_interface_request.market_service_request import MarketServiceRequest
from market_place_cli.service_interface_request.common_request import get_table_choice
from market_place_cli.service_interface_logic.common import get_net, wallet_has_password
from market_place_cli.v_cloud_market_cli_common.service_display.wallet_service_display import WalletServiceDisplay
from market_place_cli.v_cloud_market_cli_common.utils.service_error import (
    HttpNotFoundException,
    HttpBadRequestException
)


class MarketServiceLogic(object):

    def __init__(self):
        self.title = 'MarketService'
        self.console = None
        self.mr = None
        self.md = None
        self.wr = None
        self.net_type = 'M'
        self.nonce = 2
        self.password = ''
        self.main_functions = ['Get Service Provider Information', 'Make An Order']
    @property
    def Name(self):
        return self.title

    def StartLogic(self, console: Console, isTestnet: bool):
        self.console = console
        self.wr = WalletRequest(console)
        self.net_type = get_net(isTestnet)
        self.mr = MarketServiceRequest(self.console)
        self.md = MarketServiceDisplay(self.console)
        console.clear()
        while True:
            choice = MainInterface.display_service_choice(console, self.title, self.main_functions, True)
            if choice == '1':
                # request provider
                self.provider_logic()
            elif choice == '2':
                self.make_order_logic()
            elif choice.lower() == 'b':
                break

    def provider_logic(self):
        index = self.wr.get_payment_address()
        if wallet_has_password(self.net_type):
            self.password = self.wr.get_password()
        ms = MarketService(self.net_type, self.password, index)
        cur = 1
        page_size = 10
        while True:
            result = ms.get_provider_info_page(current=cur,opt={}, page_size=page_size)
            w = self.md.display_provider_page(result)
            has_next = len(result['list']) >= page_size
            choice = get_table_choice(self.console, w, has_next, {'c': '[C]ategory Search In Provider'})
            if choice == 'p' and cur > 1:
                cur -= 1
            elif choice == 'n' and has_next:
                cur += 1
            elif choice == 'c':
                provider = self.mr.get_provider_name()
                self.category_logic(provider)
            elif choice == 'e':
                break

    def category_logic(self, provider: str):
        ms = MarketService(self.net_type, self.password)

        cur = 1
        page_size = 10
        while True:
            result = ms.get_category_info_page({'provider':provider}, cur, page_size)
            w = self.md.display_category_page(result)
            has_next = len(result['list']) >= page_size
            choice = get_table_choice(self.console, w, has_next, {'t': '[T]Service Type Search In Category'})
            if choice == 'p' and cur > 1:
                cur -= 1
            elif choice == 'n' and has_next:
                cur += 1
            elif choice == 't':
                category = self.mr.get_category_name()
                self.service_type_logic(provider, category)
            elif choice == 'e':
                break

    def service_type_logic(self, provider: str, category: str):
        ms = MarketService(self.net_type, self.password)

        cur = 1
        page_size = 10
        opt = {
            'provider': provider,
            'category': category
        }
        while True:
            try:
                result = ms.get_service_info_page(opt, cur, page_size)
                w = self.md.display_service_page(result)
                has_next = len(result['list']) >= page_size
                choice = get_table_choice(self.console, w, has_next)
                if choice == 'p' and cur > 1:
                    cur -= 1
                elif choice == 'n' and has_next:
                    cur += 1
                elif choice == 'e':
                    break
            except:
                self.console.input("[bright_red]Failed to get services info. Press ENTER to continue...")
                break

    def make_order_logic(self):
        index =  self.wr.get_payment_address()
        if wallet_has_password(self.net_type):
            self.password = self.wr.get_password()
        wsd = WalletServiceDisplay(self.console)
        ws = WalletService(wsd, self.net_type, self.password)
        pub_key = ws.fetch_wallet_info(index, 'pub')

        ms = MarketService(self.net_type, self.password, index)
        mr = MarketServiceRequest(self.console)
        qr_display = QRCodeDisplay()

        service_id = self.mr.get_service_id()
        try:
            service_info = ms.get_service_info(service_id)
        except HttpNotFoundException:
            self.console.input("[bright_red]Service doesn't exist. Press ENTER to continue...")
            return
        except:
            self.console.input("[bright_red]Failed to get service info. Press ENTER to continue...")
            return

        provider_host = ms.get_provider_host(service_info['provider'])

        opts = self.mr.user_choose_options(service_info['serviceOptions'])
        price_set = ms.find_price_set(service_info['durationToPrice'], opts)
        time_period = self.mr.user_choose_duration(price_set)
        amt = self.calculate_amount(price_set, time_period['time'])

        if not ms.enough_balance(amt):
            self.console.print(f'[bright_red]Your balance in address index {self.nonce + 1} is not enough.')
            self.console.input('[bright_red]Order Creation Aborted...')
            return
        expired_date = time_period['expiredDate'] if 'expiredDate' in time_period else ''
        order_brief = ms.make_order(
            service_id,
            amt,
            opts,
            time_period['time'],
            expired_date)

        display_qr = mr.get_display_qr_code()
        self.md.display_order_brief(order_brief)

        if display_qr:
            qr_display.show_account_of_wallet(
                address=order_brief['recipient'],
                amt=order_brief['amount'],
                invoice=order_brief['id']+'-'+pub_key)
            self.console.input('Press ENTER to continue...')

    def calculate_amount(self, price_set: dict, duration: int) -> int:
        time_units = []
        index = 0
        for k in price_set['duration']:
            time_units.append(int(k))
        time_units.sort(reverse=True)
        total_len = len(time_units)

        while index < total_len - 1 and duration < time_units[index]:
            index += 1
        amt = price_set['price'] * price_set['duration'][str(time_units[index])] * duration
        return math.ceil(amt)



import sys

from rich.console import Console
from rich.panel import Panel

from market_place_cli.v_cloud_market_cli_common.service.market_service import MarketService
from market_place_cli.v_cloud_market_cli_common.service.order_service import OrderService
from market_place_cli.v_cloud_market_cli_common.service.wallet_service import WalletService
from market_place_cli.v_cloud_market_cli_common.service_display.main_interface import MainInterface
from market_place_cli.v_cloud_market_cli_common.service_display.order_service_display import OrderServiceDisplay
from market_place_cli.v_cloud_market_cli_common.service_display.wallet_service_display import WalletServiceDisplay
from market_place_cli.service_interface_logic.common import get_net, wallet_has_password
from market_place_cli.service_interface_request.wallet_service_request import WalletRequest
from market_place_cli.service_interface_request.order_service_request import OrderRequest
from market_place_cli.service_interface_request.common_request import get_table_choice



class OrderServiceLogic:

    def __init__(self):
        self.title = 'OrderService'
        self.console = None
        self.od = None  # order display instance
        self.net_type = 'M'
        self.main_functions = ['Show Pending Order', 'Show Paid Order', 'Show Filed Order',
                               'Show Refunded Order', 'Show Order Detail', 'Pay An Order',
                               'Refund Order']
        self.password = None

    @property
    def Name(self):
        return self.title

    def StartLogic(self, console: Console, is_testnet: bool):
        self.console = console
        self.net_type = get_net(is_testnet)
        self.od = OrderServiceDisplay(self.console)
        console.clear()
        while True:
            choice = MainInterface.display_service_choice(console, self.title, self.main_functions, True)
            if choice == '1':
                self.show_pending_order_logic()
            elif choice == '2':
                self.show_paid_order_logic()
            elif choice == '3':
                self.show_filed_order_logic()
            elif choice == '4':
                self.show_refunded_order_logic()
            elif choice == '5':
                self.show_order_detail_logic()
            elif choice == '6':
                self.pay_order_logic()
            elif choice == '7':
                self.show_refundable_order_logic()
            elif choice.lower() == 'b':
                break

    def show_pending_order_logic(self):
        self.show_order_page(status='OrderPending')

    def show_paid_order_logic(self):
        self.show_order_page(status='OrderPaid')

    def show_filed_order_logic(self):
        self.show_order_page(status='OrderFiled')

    def show_refunded_order_logic(self):
        self.show_order_page(status='OrderRefund')

    def show_refundable_order_logic(self):
        self.show_order_page(status='OrderRefundable')

    def show_order_detail_logic(self):
        wr = WalletRequest(self.console)
        orderReq = OrderRequest(self.console)
        index = wr.get_payment_address()
        if wallet_has_password(self.net_type):
            self.password = wr.get_self.password()
        orderService = OrderService(self.net_type, self.password, index)
        oID = orderReq.get_order_id()
        try:
            info = orderService.get_order_info(oID)
            self.od.display_order_info(info)
        except Exception as e:
            self.console.print(e)
            self.console.input('Press ENTER to continue...')

    def show_order_page(self, status: str):
        wr = WalletRequest(self.console)

        index = wr.get_payment_address()
        if wallet_has_password(self.net_type):
            self.password = wr.get_password()
        o = OrderService(self.net_type, self.password, index)
        cur = 1
        page_size = 10

        title = self._construct_page_title(status)
        extra = self._construct_page_button(status)
        while True:
            try:
                display_result = self._construct_order_page_data(o, cur, status)
            except Exception as e:
                self.console.print(e)
                self.console.input('Press ENTER to continue...')
                return
            w = self.od.display_order_page(title, display_result)
            has_next = len(display_result['list']) >= page_size or len(display_result['list']) >= page_size

            choice = get_table_choice(self.console, w, has_next, extra=extra)
            if choice == 'p' and cur > 1:
                cur -= 1
            elif choice == 'n' and has_next:
                cur += 1
            elif choice == 'e':
                break
            elif status == 'OrderRefundable' and choice == 'r':
                order_list = display_result['list']
                self.refund_order_detail_logic(index, order_list)

    def pay_order_logic(self):
        order_req = OrderRequest(self.console)
        wr = WalletRequest(self.console)
        order_id = order_req.get_order_id()
        index = wr.get_payment_address()
        if wallet_has_password(self.net_type):
            self.password = wr.get_password()
        amt = wr.get_amount()
        if amt < 0:
            self.console.print('[bright_red]!!! Invalid Amount !!!')
            self.console.input('Press ENTER to exit...')
            return
        try:
            order_service = OrderService(self.net_type, self.password, index)
            wsd = WalletServiceDisplay(self.console)
            ws = WalletService(wsd, self.net_type, self.password)
            pubKey = ws.fetch_wallet_info(index, 'pub')
            order_info = order_service.get_order_info(order_id)
            recipient = order_info['recipient']
            amt = self._overpay_protection(order_info, amt)
            ws.account_pay(index, recipient, amt, order_id+';'+pubKey)
        except Exception as e:
            self.console.print(e)
            self.console.input('[bright_red]Failed to pay for order !!!')
        # mock payment code for local testing
        # resp = order_service.mock_order_payment(order_id, recipient, pubKey, amt)
        # self.console.print(Panel.fit(resp["content"]))

    def refund_order_detail_logic(self, index: int, order_list: list):
        wr = WalletRequest(self.console)
        orderReq = OrderRequest(self.console)
        order_service = OrderService(self.net_type, self.password, index)
        order_id = orderReq.get_order_id()
        provider = [item for item in order_list if item['id'] == order_id]
        if len(provider) == 0:
            return
        ms = MarketService(self.net_type, self.password)
        provider_host = ms.get_provider_host(provider[0]['provider'])
        try:
            info = order_service.refund_order(provider_host, order_id)
        except Exception as e:
            self.console.print(e)
        self.console.input('Press ENTER to exit...')

    def _construct_order_page_data(self, o: OrderService, cur_page: int, order_status: str):
        display_result = {
            'pagination': {}
        }
        if order_status == 'OrderRefundable':
            filed_result = o.get_order_info_page(current=cur_page, status='OrderFiled')
            paid_result = o.get_order_info_page(current=cur_page, status='OrderPaid')
            display_result['list'] = paid_result['list'] + filed_result['list']
            display_result['pagination']['total'] = paid_result['pagination']['total'] + filed_result['pagination']['total']
            display_result['pagination']['current'] = paid_result['pagination']['current']
            display_result['pagination']['pageSize'] = paid_result['pagination']['pageSize']
        else:
            display_result = o.get_order_info_page(current=cur_page, status=order_status)
        return display_result

    def _construct_page_title(self, order_status: str):
        title = 'Order Information Table'
        if order_status == 'OrderPending':
            title = 'Pending ' + title
        elif order_status == 'OrderPaid':
            title = 'Paid ' + title
        elif order_status == 'OrderFiled':
            title = 'Filed ' + title
        elif order_status == 'OrderComplete':
            title = 'Completed ' + title
        elif order_status == 'OrderRefund':
            title = 'Refund ' + title
        return title

    def _construct_page_button(self, order_status: str):
        if order_status == 'OrderRefundable':
            extra = {'r': '[R]Refund'}
        else:
            extra = {}
        return extra

    def _overpay_protection(self, order_info: dict, amt: int):
        check_amt = order_info['amountPaid'] + amt
        remain_amt = order_info['amount'] > order_info['amountPaid']
        if order_info['status'] == 'OrderPending':
            if check_amt > order_info['amount'] and remain_amt > 0:
                self.console.print('[bright_red]The input amount is larger than remaining order amount !!!')
                choice = self.console.input('[bright_green]Continue the payment with remaining amount (default Y)[Y/n]: ')
                if choice.lower() == 'n':
                    return amt
                else:
                    amt = order_info['amount'] - order_info['amountPaid']
                    self.console.print('')
                    self.console.print(f'[bright_green]The payment amount will be: {amt}')
                    self.console.print('')
                    self.console.input('Press ENTER to continue...')
                    return amt
            return amt
        else:
            self.console.print(f'[bright_green]Order is already paid with status - {order_info["status"]}')
            self.console.input('Press ENTER to continue...')




import base58
import logging
from .crypto import hashChain, bytes2str, str2bytes
from .setting import ADDRESS_LENGTH, ADDRESS_CHECKSUM_LENGTH

class Chain(object):

    def __init__(self, chainName, chainId, addressVersion, apiWrapper):
        self.chainName = chainName
        self.chainId = chainId
        self.addressVersion = addressVersion
        self.apiWrapper = apiWrapper

    def get_connected_peers(self):
        response = self.apiWrapper.request('/peers/connected')
        if not response.get('peers'):
            return []
        else:
            return [peer['address'] for peer in response.get('peers')]

    def tx(self, id):
        return self.apiWrapper.request(f'/transactions/info/{id}')

    def unconfirmed_tx(self, id):
        return self.apiWrapper.request(f'/transactions/unconfirmed/info/{id}')

    def validate_address(self, address):
        addr = bytes2str(base58.b58decode(address.encode('utf-8'))) #bytes2str(base58.b58decode(address))
        if addr[0] != chr(self.addressVersion):
            logging.error("Wrong address version")
        elif addr[1] != self.chainId:
            logging.error("Wrong chain id")
        elif len(addr) != ADDRESS_LENGTH:
            logging.error("Wrong address length")
        elif addr[-ADDRESS_CHECKSUM_LENGTH:] != hashChain(str2bytes(addr[:-ADDRESS_CHECKSUM_LENGTH]))[:ADDRESS_CHECKSUM_LENGTH]:
            logging.error("Wrong address checksum")
        else:
            return True
        return False

    def public_key_to_address(self, publicKey):
        unhashedAddress = chr(self.addressVersion) + \
            str(self.chainId) + hashChain(publicKey)[0:20]
        addressHash = hashChain(str2bytes(unhashedAddress))[0:4]
        address = bytes2str(base58.b58encode(
            str2bytes(unhashedAddress + addressHash)))
        return address



from contextlib import contextmanager
from rich.console import Console
from time import time
from datetime import timezone

BEAT_TIME = 0.02


@contextmanager
def beat(console: Console, length: int = 1) -> None:
    with console:
        console.clear()
        yield
    time.sleep(length * BEAT_TIME)


def utc_to_local(utc_dt):
    return utc_dt.replace(tzinfo=timezone.utc).astimezone(tz=None)

def local_to_utc(local_dt):
    pass



from rich.console import Console
from rich.table import Table
from rich import box

def display_table(console:Console, title: str, headers: list, data: list):
    if len(data) == 0:
        console.print("No data!")
        return
    table = Table(show_header=True)
    table.title = title
    table.box = box.ROUNDED

    for header in headers:
        cell_justify = header.get("justify", "center")
        table.add_column(header["text"], justify=cell_justify)

    for item in data:
        row_data = dict_to_str_array(headers, item)
        table.add_row(*row_data)

    console.print(table)

def dict_to_str_array(headers: list, data: dict) -> list:
        keys = data.keys()
        result_list = []
        if len(keys) == 0:
            return result_list
        for header in headers:
            key = header["value"]
            if key not in data:
                result_list.append("-")
                continue
            if type(data[key]) is dict:
                result_list.append(dict_to_string(data[key]))
            elif type(data[key]) is list:
                result_list.append(list_to_string(data[key]))
            else:
                result_list.append(f'{data[key]}')
        return result_list

def dict_to_string(data: dict) -> str:
    if data.keys() == 0:
        return "-"
    result = ""
    for key in data:
        if type(data[key]) in (int, str, bool, float):
            result += f'{key}: {data[key]}\n'
        elif type(data[key]) is list:
            result += list_to_string(data[key])
    return result

def list_to_string(item_list: list) -> str:
    if len(item_list) == 0:
        return ""
    if type(item_list[0]) == str:
       return  ", ".join(item_list)
    result = ""
    for item in item_list:
        item_str = dict_to_string(item)
        result += item_str
    return result
    


import os, sys
sys.path.append(os.getcwd())

import click
from enum import Enum
from rich.console import Console

from market_place_cli.v_cloud_market_cli_common.service_display.main_interface import MainInterface
from market_place_cli.service_interface_logic import Logics


class ServiceState(Enum):
    MainService = 0
    MarketService = 1
    OrderService = 2
    WalletService = 3
    UserService = 4
    UpperLevelService = 5


@click.command()
@click.option('-t', '--testnet', is_flag=True, help='Specify net type')
def start(testnet):
    console = Console()
    curState = ServiceState.MainService
    Logics.container['Wallet Initialize'].StartLogic(console, testnet)
    Logics.container.pop('Wallet Initialize', None)
    while True:
        try:
            state = int(service_execution(console, curState, testnet))
            if state == ServiceState.UpperLevelService.value:
                curState = ServiceState.MainService
            elif state == ServiceState.MarketService.value:
                curState = ServiceState.MarketService
            elif state == ServiceState.OrderService.value:
                curState = ServiceState.OrderService
            elif state == ServiceState.WalletService.value:
                curState = ServiceState.WalletService
            elif state == ServiceState.UserService.value:
                curState = ServiceState.UserService
            else:
                print('You should not be here...')
        except Exception:
            pass


def service_execution(console: Console, state: ServiceState, isTestnet: bool) -> str:
    if state == ServiceState.MainService:
        newState = MainInterface.display_service_choice(console, "Main Services", Logics.container.keys())
        if newState == '' or int(newState) > ServiceState.UpperLevelService.value or int(newState) < ServiceState.MainService.value:
            return service_execution(console, state, isTestnet)
        return newState
    elif state == ServiceState.MarketService:
        Logics.container[ServiceState.MarketService.name].StartLogic(console, isTestnet)
        return str(ServiceState.UpperLevelService.value)
    elif state == ServiceState.OrderService:
        Logics.container[ServiceState.OrderService.name].StartLogic(console, isTestnet)
        return str(ServiceState.UpperLevelService.value)
    elif state == ServiceState.WalletService:
        Logics.container[ServiceState.WalletService.name].StartLogic(console, isTestnet)
        return str(ServiceState.UpperLevelService.value)
    elif state == ServiceState.UserService:
        Logics.container[ServiceState.UserService.name].StartLogic(console, isTestnet)
        return str(ServiceState.UpperLevelService.value)
    else:
        console.input('You should not be here...')



from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from rich.columns import Columns
from rich.padding import Padding

def get_table_choice(console: Console, w: int, has_next: bool, extra={}) -> str:
    options = ['[P]rev', '[E]xit']
    basic_choice = ['p', 'e']
    if has_next:
        options.append('[N]ext')
    if extra:
        extra_choices = list(extra.keys())
        basic_choice.extend(extra_choices)
        for key in extra_choices:
            options.append(extra[key])

    max_opt_len = 0
    for index, val in enumerate(options):
        max_opt_len = max(max_opt_len, len(val))
        options[index] =  "[bright_green]" + val

    # console.width is console width of charater
    # one character space for panel outline , two space for column padding
    column_width = int(console.width / 2) - 3
    padding_width = int((column_width - max_opt_len) / 2)

    cols = []
    for option in options:
        cols.append(Padding(option, (0, padding_width)))

    p = Panel.fit(Columns(cols, width=column_width))
    console.print(p, justify='center')

    while True:
        choice = console.input('[green]navigate to: ')
        if choice.lower() in basic_choice:
            return choice.lower()
        elif has_next and choice.lower() in ['n']:
            return choice.lower()



from market_place_cli.v_cloud_market_cli_common.service.wallet_service import WalletService
from market_place_cli.v_cloud_market_cli_common.utils.service_error import WalletStorageLoadingException


def get_net(is_testnet) -> str:
    return 'T' if is_testnet else 'M'


def wallet_has_password(net_type: str) -> bool:
    try:
        ws = WalletService(None, net_type, "", show_err=False)
        return ws.wallet_data is None
    except WalletStorageLoadingException:
        return True



import logging
import requests
from requests.exceptions import RequestException


class Wrapper(object):

    def __init__(self, nodeHost, apiKey=''):
        self.nodeHost = nodeHost
        self.apiKey = apiKey

    def request(self, api, postData=''):
        headers = {}
        url = self.nodeHost + api
        if self.apiKey:
            headers['api_key'] = self.apiKey
        headerStr = ' '.join(['--header \'{}: {}\''.format(k, v) for k, v in headers.items()])
        try:
            if postData:
                headers['Content-Type'] = 'application/json'
                dataStr = '-d {}'.format(postData)
                logging.info(f'curl -X POST {headerStr} {dataStr} {url}')
                resp = requests.post(url, data=postData, headers=headers)
                print(f'curl -X POST {headerStr} {dataStr} {url}')
                return resp.json()
            else:
                logging.info(f'curl -X GET {headerStr} {url}')
                return requests.get(url, headers=headers).json()
        except RequestException as e:
            msg = f'Failed to get response: {e}'
            raise Exception(msg)



import os
import json
from collections import namedtuple
from .vsyschain.error import throw_error
from .service_error import (
    WalletStorageLoadingException,
    WalletStorageSavingException)

WalletData = namedtuple("WalletData", ["seed", "accountSeeds", "nonce", "agent"])

class WalletStorage:

    def __init__(self, path = None):
        self.path = path

    def save(self, wallet_data, cipher=None):
        """
        :param wallet_data: a json dict
        :param cipher: an instance of cipher class for encrypting json data
        :return: None
        """
        try:
            parentPath = os.path.dirname(self.path)
            if not os.path.exists(parentPath):
                os.mkdir(parentPath)
            data = json.dumps(wallet_data._asdict())
            with open(self.path, "w") as wallet:
                if cipher:
                    wallet.write(cipher.encrypt(data))
                else:
                    wallet.write(data)
        except:
            msg = "Failed to save information to wallet file."
            throw_error(msg, WalletStorageSavingException)

    def load(self, cipher=None, show_err=True):
        """
        :param cipher: an instance of cipher class, default: None
        :return: a json dict, walletData, with keys: seed, accountSeeds, nonce, agent
        """
        try:
            with open(self.path, "r") as wallet:
                data = wallet.read()
                # cannot open wallet saved as plain text with cipher
                if cipher:
                    plain_data = cipher.decrypt(data)
                    return json.loads(plain_data, object_hook=self._wallet_data_decoder)
                else:
                    return json.loads(data, object_hook=self._wallet_data_decoder)
        except FileNotFoundError as e:
            return None
        except:
            if show_err:
                msg = "Failed to load information from wallet file."
                throw_error(msg, WalletStorageLoadingException)

    def _wallet_data_decoder(self, walletDict):
        return namedtuple("WalletData", walletDict.keys())(*walletDict.values())

    def is_json(self,input):
        try:
            json.load(input)
            return True
        except:
            return False



from rich.console import Console
from rich.prompt import Prompt


class WalletRequest:

    def __init__(self, console: Console):
        self.console = console

    def get_num_address(self) -> int:
        msg = '[green]Please enter number of address in wallet (1-10): '
        num = self._get_num(msg)
        return num

    def get_payment_address(self) -> int:
        msg = 'Please enter the index of address that you want to use: '
        return self._get_num(msg) - 1

    def get_password(self) -> str:
        msg = '[green]Please enter password for encrypting wallet (Press ENTER for no encryption): '
        return Prompt.ask(msg, password=True)

    def get_dec_password(self) -> str:
        msg = '[green]Please enter password for decrypting wallet: '
        return Prompt.ask(msg, password=True)

    def get_seed(self) -> str:
        msg = '[green]Please enter your wallet seed: '
        return self._get_input(msg)

    def save_to_csv(self) -> bool:
        msg = '[green]Save generated addresses to a csv file (default n) [Y/n]: '
        choice = self._get_input(msg)
        return self.get_yes_no(choice)

    def display_detail_balance(self) -> bool:
        msg = '[green]Display detailed balance info (default n) [Y/n]: '
        choice = self._get_input(msg)
        return self.get_yes_no(choice)

    def get_to_append(self) -> bool:
        msg = '[green]Append new generated addresses to wallet (default n) [Y/n]: '
        choice = self._get_input(msg)
        return self.get_yes_no(choice)

    def get_recipient_address(self) -> str:
        msg = '[green]Please enter recipient address: '
        return self._get_input(msg)

    def get_amount(self) -> int:
        msg = '[green]Please enter amount: '
        amt = self._get_input(msg)
        if not amt.isnumeric() or int(amt) < 0:
            self.console.print('[bright_red]Invalid input amount !')
            return -1
        return int(amt)

    def get_attachment(self) -> str:
        msg = '[green]Please enter attachment for this payment[/] [optional]: '
        return self._get_input(msg)

    def get_yes_no(self, choice: str) -> bool:
        if choice.lower() not in ['y', 'n', '']:
            choice = 'n'
        if choice == '':
            choice = 'n'
        return choice.lower() == 'y'

    def _get_num(self, msg) -> int:
        num = 1
        while True:
            try:
                num = int(self.console.input(msg))
                if num < 1 or num > 10:
                    raise ValueError
            except ValueError:
                self.console.print('[bold bright_red]!!! Invalid Input Number !!!')
                continue
            break
        return num

    def _get_input(self, msg: str) -> str:
        return self.console.input(msg)




import os
import time
from rich.console import Console

from market_place_cli.service_interface_request.wallet_service_request import WalletRequest
from market_place_cli.v_cloud_market_cli_common.config.wallet_config import WALLET_FILENAME
from market_place_cli.v_cloud_market_cli_common.service.wallet_service import WalletService
from market_place_cli.v_cloud_market_cli_common.service_display.main_interface import MainInterface
from market_place_cli.v_cloud_market_cli_common.service_display.wallet_service_display import WalletServiceDisplay


class InitializationLogic(object):

    def __init__(self):
        # TODO use wallet storage
        self.title = 'Wallet Initialize'
        self.console = None
        self.isTestnet = False
        self.main_functions = ['Recover Wallet From Seed', 'Generate New Wallet']

    @property
    def Name(self):
        return self.title

    def StartLogic(self, console: Console, isTestnet: bool):
        # This should be executed once if no wallet file detected
        self.console = console
        self.isTestnet = isTestnet
        console.clear()
        if os.path.isfile(WALLET_FILENAME):
            return
        MainInterface.display_title(console, self.title)
        console.print('[red]System does not detect wallet in your local environment.[/]')
        time.sleep(2)
        while True:
            choice = MainInterface.display_service_choice(console, self.title, self.main_functions)
            if choice not in ['1', '2']:
                console.print('[red] !!! Invalid Choice !!!')
                time.sleep(2)
                continue
            break
        if choice == '1':
            self._recover_from_seed()
        elif choice == '2':
            self._generate_new_wallet()

    def _recover_from_seed(self):
        wr = WalletRequest(self.console)
        numAddr = wr.get_num_address()
        net = 'T' if self.isTestnet else 'M'
        password = wr.get_password()
        wsd = WalletServiceDisplay(self.console)
        ws = WalletService(wsd, net, password)
        seed = wr.get_seed()
        ws.recover_wallet(seed, numAddr, wr.save_to_csv())

    def _generate_new_wallet(self):
        wr = WalletRequest(self.console)
        numAddr = wr.get_num_address()
        net = 'T' if self.isTestnet else 'M'
        password = wr.get_password()
        wsd = WalletServiceDisplay(self.console)
        ws = WalletService(wsd, net, password)
        ws.seed_generate(wr.save_to_csv(),
                         wr.display_detail_balance(),
                         numAddr)



from nacl.public import PrivateKey, SealedBox
import base58


def decrypt_message(public_key, msg):
    private_key = PrivateKey(base58.b58decode(public_key))
    box = SealedBox(private_key)
    return box.decrypt(base58.b58decode(msg)).decode('UTF-8')



WORD_LIST = ["abandon", "ability", "able", "about", "above", "absent", "absorb", "abstract", "absurd", "abuse", "access",
      "accident", "account", "accuse", "achieve", "acid", "acoustic", "acquire", "across", "act", "action",
      "actor", "actress", "actual", "adapt", "add", "addict", "address", "adjust", "admit", "adult", "advance",
      "advice", "aerobic", "affair", "afford", "afraid", "again", "age", "agent", "agree", "ahead", "aim", "air",
      "airport", "aisle", "alarm", "album", "alcohol", "alert", "alien", "all", "alley", "allow", "almost",
      "alone", "alpha", "already", "also", "alter", "always", "amateur", "amazing", "among", "amount", "amused",
      "analyst", "anchor", "ancient", "anger", "angle", "angry", "animal", "ankle", "announce", "annual",
      "another", "answer", "antenna", "antique", "anxiety", "any", "apart", "apology", "appear", "apple",
      "approve", "april", "arch", "arctic", "area", "arena", "argue", "arm", "armed", "armor", "army", "around",
      "arrange", "arrest", "arrive", "arrow", "art", "artefact", "artist", "artwork", "ask", "aspect", "assault",
      "asset", "assist", "assume", "asthma", "athlete", "atom", "attack", "attend", "attitude", "attract",
      "auction", "audit", "august", "aunt", "author", "auto", "autumn", "average", "avocado", "avoid", "awake",
      "aware", "away", "awesome", "awful", "awkward", "axis", "baby", "bachelor", "bacon", "badge", "bag",
      "balance", "balcony", "ball", "bamboo", "banana", "banner", "bar", "barely", "bargain", "barrel", "base",
      "basic", "basket", "battle", "beach", "bean", "beauty", "because", "become", "beef", "before", "begin",
      "behave", "behind", "believe", "below", "belt", "bench", "benefit", "best", "betray", "better", "between",
      "beyond", "bicycle", "bid", "bike", "bind", "biology", "bird", "birth", "bitter", "black", "blade", "blame",
      "blanket", "blast", "bleak", "bless", "blind", "blood", "blossom", "blouse", "blue", "blur", "blush",
      "board", "boat", "body", "boil", "bomb", "bone", "bonus", "book", "boost", "border", "boring", "borrow",
      "boss", "bottom", "bounce", "box", "boy", "bracket", "brain", "brand", "brass", "brave", "bread", "breeze",
      "brick", "bridge", "brief", "bright", "bring", "brisk", "broccoli", "broken", "bronze", "broom", "brother",
      "brown", "brush", "bubble", "buddy", "budget", "buffalo", "build", "bulb", "bulk", "bullet", "bundle",
      "bunker", "burden", "burger", "burst", "bus", "business", "busy", "butter", "buyer", "buzz", "cabbage",
      "cabin", "cable", "cactus", "cage", "cake", "call", "calm", "camera", "camp", "can", "canal", "cancel",
      "candy", "cannon", "canoe", "canvas", "canyon", "capable", "capital", "captain", "car", "carbon", "card",
      "cargo", "carpet", "carry", "cart", "case", "cash", "casino", "castle", "casual", "cat", "catalog", "catch",
      "category", "cattle", "caught", "cause", "caution", "cave", "ceiling", "celery", "cement", "census",
      "century", "cereal", "certain", "chair", "chalk", "champion", "change", "chaos", "chapter", "charge",
      "chase", "chat", "cheap", "check", "cheese", "chef", "cherry", "chest", "chicken", "chief", "child",
      "chimney", "choice", "choose", "chronic", "chuckle", "chunk", "churn", "cigar", "cinnamon", "circle",
      "citizen", "city", "civil", "claim", "clap", "clarify", "claw", "clay", "clean", "clerk", "clever", "click",
      "client", "cliff", "climb", "clinic", "clip", "clock", "clog", "close", "cloth", "cloud", "clown", "club",
      "clump", "cluster", "clutch", "coach", "coast", "coconut", "code", "coffee", "coil", "coin", "collect",
      "color", "column", "combine", "come", "comfort", "comic", "common", "company", "concert", "conduct",
      "confirm", "congress", "connect", "consider", "control", "convince", "cook", "cool", "copper", "copy",
      "coral", "core", "corn", "correct", "cost", "cotton", "couch", "country", "couple", "course", "cousin",
      "cover", "coyote", "crack", "cradle", "craft", "cram", "crane", "crash", "crater", "crawl", "crazy",
      "cream", "credit", "creek", "crew", "cricket", "crime", "crisp", "critic", "crop", "cross", "crouch",
      "crowd", "crucial", "cruel", "cruise", "crumble", "crunch", "crush", "cry", "crystal", "cube", "culture",
      "cup", "cupboard", "curious", "current", "curtain", "curve", "cushion", "custom", "cute", "cycle", "dad",
      "damage", "damp", "dance", "danger", "daring", "dash", "daughter", "dawn", "day", "deal", "debate",
      "debris", "decade", "december", "decide", "decline", "decorate", "decrease", "deer", "defense", "define",
      "defy", "degree", "delay", "deliver", "demand", "demise", "denial", "dentist", "deny", "depart", "depend",
      "deposit", "depth", "deputy", "derive", "describe", "desert", "design", "desk", "despair", "destroy",
      "detail", "detect", "develop", "device", "devote", "diagram", "dial", "diamond", "diary", "dice", "diesel",
      "diet", "differ", "digital", "dignity", "dilemma", "dinner", "dinosaur", "direct", "dirt", "disagree",
      "discover", "disease", "dish", "dismiss", "disorder", "display", "distance", "divert", "divide", "divorce",
      "dizzy", "doctor", "document", "dog", "doll", "dolphin", "domain", "donate", "donkey", "donor", "door",
      "dose", "double", "dove", "draft", "dragon", "drama", "drastic", "draw", "dream", "dress", "drift", "drill",
      "drink", "drip", "drive", "drop", "drum", "dry", "duck", "dumb", "dune", "during", "dust", "dutch", "duty",
      "dwarf", "dynamic", "eager", "eagle", "early", "earn", "earth", "easily", "east", "easy", "echo", "ecology",
      "economy", "edge", "edit", "educate", "effort", "egg", "eight", "either", "elbow", "elder", "electric",
      "elegant", "element", "elephant", "elevator", "elite", "else", "embark", "embody", "embrace", "emerge",
      "emotion", "employ", "empower", "empty", "enable", "enact", "end", "endless", "endorse", "enemy", "energy",
      "enforce", "engage", "engine", "enhance", "enjoy", "enlist", "enough", "enrich", "enroll", "ensure",
      "enter", "entire", "entry", "envelope", "episode", "equal", "equip", "era", "erase", "erode", "erosion",
      "error", "erupt", "escape", "essay", "essence", "estate", "eternal", "ethics", "evidence", "evil", "evoke",
      "evolve", "exact", "example", "excess", "exchange", "excite", "exclude", "excuse", "execute", "exercise",
      "exhaust", "exhibit", "exile", "exist", "exit", "exotic", "expand", "expect", "expire", "explain", "expose",
      "express", "extend", "extra", "eye", "eyebrow", "fabric", "face", "faculty", "fade", "faint", "faith",
      "fall", "false", "fame", "family", "famous", "fan", "fancy", "fantasy", "farm", "fashion", "fat", "fatal",
      "father", "fatigue", "fault", "favorite", "feature", "february", "federal", "fee", "feed", "feel", "female",
      "fence", "festival", "fetch", "fever", "few", "fiber", "fiction", "field", "figure", "file", "film",
      "filter", "final", "find", "fine", "finger", "finish", "fire", "firm", "first", "fiscal", "fish", "fit",
      "fitness", "fix", "flag", "flame", "flash", "flat", "flavor", "flee", "flight", "flip", "float", "flock",
      "floor", "flower", "fluid", "flush", "fly", "foam", "focus", "fog", "foil", "fold", "follow", "food",
      "foot", "force", "forest", "forget", "fork", "fortune", "forum", "forward", "fossil", "foster", "found",
      "fox", "fragile", "frame", "frequent", "fresh", "friend", "fringe", "frog", "front", "frost", "frown",
      "frozen", "fruit", "fuel", "fun", "funny", "furnace", "fury", "future", "gadget", "gain", "galaxy",
      "gallery", "game", "gap", "garage", "garbage", "garden", "garlic", "garment", "gas", "gasp", "gate",
      "gather", "gauge", "gaze", "general", "genius", "genre", "gentle", "genuine", "gesture", "ghost", "giant",
      "gift", "giggle", "ginger", "giraffe", "girl", "give", "glad", "glance", "glare", "glass", "glide",
      "glimpse", "globe", "gloom", "glory", "glove", "glow", "glue", "goat", "goddess", "gold", "good", "goose",
      "gorilla", "gospel", "gossip", "govern", "gown", "grab", "grace", "grain", "grant", "grape", "grass",
      "gravity", "great", "green", "grid", "grief", "grit", "grocery", "group", "grow", "grunt", "guard", "guess",
      "guide", "guilt", "guitar", "gun", "gym", "habit", "hair", "half", "hammer", "hamster", "hand", "happy",
      "harbor", "hard", "harsh", "harvest", "hat", "have", "hawk", "hazard", "head", "health", "heart", "heavy",
      "hedgehog", "height", "hello", "helmet", "help", "hen", "hero", "hidden", "high", "hill", "hint", "hip",
      "hire", "history", "hobby", "hockey", "hold", "hole", "holiday", "hollow", "home", "honey", "hood", "hope",
      "horn", "horror", "horse", "hospital", "host", "hotel", "hour", "hover", "hub", "huge", "human", "humble",
      "humor", "hundred", "hungry", "hunt", "hurdle", "hurry", "hurt", "husband", "hybrid", "ice", "icon", "idea",
      "identify", "idle", "ignore", "ill", "illegal", "illness", "image", "imitate", "immense", "immune",
      "impact", "impose", "improve", "impulse", "inch", "include", "income", "increase", "index", "indicate",
      "indoor", "industry", "infant", "inflict", "inform", "inhale", "inherit", "initial", "inject", "injury",
      "inmate", "inner", "innocent", "input", "inquiry", "insane", "insect", "inside", "inspire", "install",
      "intact", "interest", "into", "invest", "invite", "involve", "iron", "island", "isolate", "issue", "item",
      "ivory", "jacket", "jaguar", "jar", "jazz", "jealous", "jeans", "jelly", "jewel", "job", "join", "joke",
      "journey", "joy", "judge", "juice", "jump", "jungle", "junior", "junk", "just", "kangaroo", "keen", "keep",
      "ketchup", "key", "kick", "kid", "kidney", "kind", "kingdom", "kiss", "kit", "kitchen", "kite", "kitten",
      "kiwi", "knee", "knife", "knock", "know", "lab", "label", "labor", "ladder", "lady", "lake", "lamp",
      "language", "laptop", "large", "later", "latin", "laugh", "laundry", "lava", "law", "lawn", "lawsuit",
      "layer", "lazy", "leader", "leaf", "learn", "leave", "lecture", "left", "leg", "legal", "legend", "leisure",
      "lemon", "lend", "length", "lens", "leopard", "lesson", "letter", "level", "liar", "liberty", "library",
      "license", "life", "lift", "light", "like", "limb", "limit", "link", "lion", "liquid", "list", "little",
      "live", "lizard", "load", "loan", "lobster", "local", "lock", "logic", "lonely", "long", "loop", "lottery",
      "loud", "lounge", "love", "loyal", "lucky", "luggage", "lumber", "lunar", "lunch", "luxury", "lyrics",
      "machine", "mad", "magic", "magnet", "maid", "mail", "main", "major", "make", "mammal", "man", "manage",
      "mandate", "mango", "mansion", "manual", "maple", "marble", "march", "margin", "marine", "market",
      "marriage", "mask", "mass", "master", "match", "material", "math", "matrix", "matter", "maximum", "maze",
      "meadow", "mean", "measure", "meat", "mechanic", "medal", "media", "melody", "melt", "member", "memory",
      "mention", "menu", "mercy", "merge", "merit", "merry", "mesh", "message", "metal", "method", "middle",
      "midnight", "milk", "million", "mimic", "mind", "minimum", "minor", "minute", "miracle", "mirror", "misery",
      "miss", "mistake", "mix", "mixed", "mixture", "mobile", "model", "modify", "mom", "moment", "monitor",
      "monkey", "monster", "month", "moon", "moral", "more", "morning", "mosquito", "mother", "motion", "motor",
      "mountain", "mouse", "move", "movie", "much", "muffin", "mule", "multiply", "muscle", "museum", "mushroom",
      "music", "must", "mutual", "myself", "mystery", "myth", "naive", "name", "napkin", "narrow", "nasty",
      "nation", "nature", "near", "neck", "need", "negative", "neglect", "neither", "nephew", "nerve", "nest",
      "net", "network", "neutral", "never", "news", "next", "nice", "night", "noble", "noise", "nominee",
      "noodle", "normal", "north", "nose", "notable", "note", "nothing", "notice", "novel", "now", "nuclear",
      "number", "nurse", "nut", "oak", "obey", "object", "oblige", "obscure", "observe", "obtain", "obvious",
      "occur", "ocean", "october", "odor", "off", "offer", "office", "often", "oil", "okay", "old", "olive",
      "olympic", "omit", "once", "one", "onion", "online", "only", "open", "opera", "opinion", "oppose",
      "option", "orange", "orbit", "orchard", "order", "ordinary", "organ", "orient", "original", "orphan",
      "ostrich", "other", "outdoor", "outer", "output", "outside", "oval", "oven", "over", "own", "owner",
      "oxygen", "oyster", "ozone", "pact", "paddle", "page", "pair", "palace", "palm", "panda", "panel", "panic",
      "panther", "paper", "parade", "parent", "park", "parrot", "party", "pass", "patch", "path", "patient",
      "patrol", "pattern", "pause", "pave", "payment", "peace", "peanut", "pear", "peasant", "pelican", "pen",
      "penalty", "pencil", "people", "pepper", "perfect", "permit", "person", "pet", "phone", "photo", "phrase",
      "physical", "piano", "picnic", "picture", "piece", "pig", "pigeon", "pill", "pilot", "pink", "pioneer",
      "pipe", "pistol", "pitch", "pizza", "place", "planet", "plastic", "plate", "play", "please", "pledge",
      "pluck", "plug", "plunge", "poem", "poet", "point", "polar", "pole", "police", "pond", "pony", "pool",
      "popular", "portion", "position", "possible", "post", "potato", "pottery", "poverty", "powder", "power",
      "practice", "praise", "predict", "prefer", "prepare", "present", "pretty", "prevent", "price", "pride",
      "primary", "print", "priority", "prison", "private", "prize", "problem", "process", "produce", "profit",
      "program", "project", "promote", "proof", "property", "prosper", "protect", "proud", "provide", "public",
      "pudding", "pull", "pulp", "pulse", "pumpkin", "punch", "pupil", "puppy", "purchase", "purity", "purpose",
      "purse", "push", "put", "puzzle", "pyramid", "quality", "quantum", "quarter", "question", "quick", "quit",
      "quiz", "quote", "rabbit", "raccoon", "race", "rack", "radar", "radio", "rail", "rain", "raise", "rally",
      "ramp", "ranch", "random", "range", "rapid", "rare", "rate", "rather", "raven", "raw", "razor", "ready",
      "real", "reason", "rebel", "rebuild", "recall", "receive", "recipe", "record", "recycle", "reduce",
      "reflect", "reform", "refuse", "region", "regret", "regular", "reject", "relax", "release", "relief",
      "rely", "remain", "remember", "remind", "remove", "render", "renew", "rent", "reopen", "repair", "repeat",
      "replace", "report", "require", "rescue", "resemble", "resist", "resource", "response", "result", "retire",
      "retreat", "return", "reunion", "reveal", "review", "reward", "rhythm", "rib", "ribbon", "rice", "rich",
      "ride", "ridge", "rifle", "right", "rigid", "ring", "riot", "ripple", "risk", "ritual", "rival", "river",
      "road", "roast", "robot", "robust", "rocket", "romance", "roof", "rookie", "room", "rose", "rotate",
      "rough", "round", "route", "royal", "rubber", "rude", "rug", "rule", "run", "runway", "rural", "sad",
      "saddle", "sadness", "safe", "sail", "salad", "salmon", "salon", "salt", "salute", "same", "sample", "sand",
      "satisfy", "satoshi", "sauce", "sausage", "save", "say", "scale", "scan", "scare", "scatter", "scene",
      "scheme", "school", "science", "scissors", "scorpion", "scout", "scrap", "screen", "script", "scrub", "sea",
      "search", "season", "seat", "second", "secret", "section", "security", "seed", "seek", "segment", "select",
      "sell", "seminar", "senior", "sense", "sentence", "series", "service", "session", "settle", "setup",
      "seven", "shadow", "shaft", "shallow", "share", "shed", "shell", "sheriff", "shield", "shift", "shine",
      "ship", "shiver", "shock", "shoe", "shoot", "shop", "short", "shoulder", "shove", "shrimp", "shrug",
      "shuffle", "shy", "sibling", "sick", "side", "siege", "sight", "sign", "silent", "silk", "silly", "silver",
      "similar", "simple", "since", "sing", "siren", "sister", "situate", "six", "size", "skate", "sketch", "ski",
      "skill", "skin", "skirt", "skull", "slab", "slam", "sleep", "slender", "slice", "slide", "slight", "slim",
      "slogan", "slot", "slow", "slush", "small", "smart", "smile", "smoke", "smooth", "snack", "snake", "snap",
      "sniff", "snow", "soap", "soccer", "social", "sock", "soda", "soft", "solar", "soldier", "solid",
      "solution", "solve", "someone", "song", "soon", "sorry", "sort", "soul", "sound", "soup", "source", "south",
      "space", "spare", "spatial", "spawn", "speak", "special", "speed", "spell", "spend", "sphere", "spice",
      "spider", "spike", "spin", "spirit", "split", "spoil", "sponsor", "spoon", "sport", "spot", "spray",
      "spread", "spring", "spy", "square", "squeeze", "squirrel", "stable", "stadium", "staff", "stage", "stairs",
      "stamp", "stand", "start", "state", "stay", "steak", "steel", "stem", "step", "stereo", "stick", "still",
      "sting", "stock", "stomach", "stone", "stool", "story", "stove", "strategy", "street", "strike", "strong",
      "struggle", "student", "stuff", "stumble", "style", "subject", "submit", "subway", "success", "such",
      "sudden", "suffer", "sugar", "suggest", "suit", "summer", "sun", "sunny", "sunset", "super", "supply",
      "supreme", "sure", "surface", "surge", "surprise", "surround", "survey", "suspect", "sustain", "swallow",
      "swamp", "swap", "swarm", "swear", "sweet", "swift", "swim", "swing", "switch", "sword", "symbol",
      "symptom", "syrup", "system", "table", "tackle", "tag", "tail", "talent", "talk", "tank", "tape", "target",
      "task", "taste", "tattoo", "taxi", "teach", "team", "tell", "ten", "tenant", "tennis", "tent", "term",
      "test", "text", "thank", "that", "theme", "then", "theory", "there", "they", "thing", "this", "thought",
      "three", "thrive", "throw", "thumb", "thunder", "ticket", "tide", "tiger", "tilt", "timber", "time", "tiny",
      "tip", "tired", "tissue", "title", "toast", "tobacco", "today", "toddler", "toe", "together", "toilet",
      "token", "tomato", "tomorrow", "tone", "tongue", "tonight", "tool", "tooth", "top", "topic", "topple",
      "torch", "tornado", "tortoise", "toss", "total", "tourist", "toward", "tower", "town", "toy", "track",
      "trade", "traffic", "tragic", "train", "transfer", "trap", "trash", "travel", "tray", "treat", "tree",
      "trend", "trial", "tribe", "trick", "trigger", "trim", "trip", "trophy", "trouble", "truck", "true",
      "truly", "trumpet", "trust", "truth", "try", "tube", "tuition", "tumble", "tuna", "tunnel", "turkey",
      "turn", "turtle", "twelve", "twenty", "twice", "twin", "twist", "two", "type", "typical", "ugly",
      "umbrella", "unable", "unaware", "uncle", "uncover", "under", "undo", "unfair", "unfold", "unhappy",
      "uniform", "unique", "unit", "universe", "unknown", "unlock", "until", "unusual", "unveil", "update",
      "upgrade", "uphold", "upon", "upper", "upset", "urban", "urge", "usage", "use", "used", "useful", "useless",
      "usual", "utility", "vacant", "vacuum", "vague", "valid", "valley", "valve", "van", "vanish", "vapor",
      "various", "vast", "vault", "vehicle", "velvet", "vendor", "venture", "venue", "verb", "verify", "version",
      "very", "vessel", "veteran", "viable", "vibrant", "vicious", "victory", "video", "view", "village",
      "vintage", "violin", "virtual", "virus", "visa", "visit", "visual", "vital", "vivid", "vocal", "voice",
      "void", "volcano", "volume", "vote", "voyage", "wage", "wagon", "wait", "walk", "wall", "walnut", "want",
      "warfare", "warm", "warrior", "wash", "wasp", "waste", "water", "wave", "way", "wealth", "weapon", "wear",
      "weasel", "weather", "web", "wedding", "weekend", "weird", "welcome", "west", "wet", "whale", "what",
      "wheat", "wheel", "when", "where", "whip", "whisper", "wide", "width", "wife", "wild", "will", "win",
      "window", "wine", "wing", "wink", "winner", "winter", "wire", "wisdom", "wise", "wish", "witness", "wolf",
      "woman", "wonder", "wood", "wool", "word", "work", "world", "worry", "worth", "wrap", "wreck", "wrestle",
      "wrist", "write", "wrong", "yard", "year", "yellow", "you", "young", "youth", "zebra", "zero", "zone", "zoo"]

ADDRESS_CSV_FILENAME = "addresses.csv"
WALLET_FILENAME = "wallet.dat"
WALLET_SPECIFICATION = "1.0"
AGENT_NAME = "V Cloud Command Tool"
AGENT_VERSION = "0.1.0"
AGENT_STRING = "V Systems Wallet Specification:" + WALLET_SPECIFICATION + "/" + AGENT_NAME + ":" + AGENT_VERSION

NO_DISPLAY = 0
DISPLAY_BALANCE = 1
DISPLAY_BALANCE_DETAIL = 2



from rich.console import Console
from rich.prompt import Prompt

from market_place_cli.v_cloud_market_cli_common.service_display.wallet_service_display import WalletServiceDisplay
from market_place_cli.v_cloud_market_cli_common.service_display.main_interface import MainInterface
from market_place_cli.v_cloud_market_cli_common.service.wallet_service import WalletService
from market_place_cli.v_cloud_market_cli_common.utils.service_error import WalletStorageLoadingException
from market_place_cli.service_interface_request.wallet_service_request import WalletRequest
from market_place_cli.service_interface_logic.common import get_net, wallet_has_password


class WalletServiceLogic(object):

    def __init__(self):
        self.title = 'WalletService'
        self.console = None
        self.net_type = 'M'
        self.main_functions = ['Show Address', 'Make Payment to An Address', 'Generate New Address',
                               'Reset Current Wallet', 'Set Wallet Password']

    @property
    def Name(self):
        return self.title

    def StartLogic(self, console: Console, isTestnet: bool):
        self.console = console
        self.wr = WalletRequest(console)
        self.wsd = WalletServiceDisplay(self.console)
        self.net_type = get_net(isTestnet)
        console.clear()
        while True:
            choice = MainInterface.display_service_choice(console, self.title, self.main_functions, True)
            if choice == '1':
                self.show_address_logic()
            elif choice == '2':
                self.make_payment_logic()
            elif choice == '3':
                self.generate_new_address_logic()
            elif choice == '4':
                self.reset_wallet_logic()
            elif choice == '5':
                self.set_wallet_password_logic()
            elif choice.lower() == 'b':
                break

    def show_address_logic(self):
        try:
            MainInterface.display_title(self.console, self.title)
            password = ''
            if wallet_has_password(self.net_type):
                password = self.wr.get_dec_password()

            ws = WalletService(self.wsd, self.net_type, password)
            ws.show_address(self.wr.save_to_csv(), self.wr.display_detail_balance())
        except Exception as e:
            print(e)
            self.console.input("enter to exist")

    def reset_wallet_logic(self):
        MainInterface.display_title(self.console, self.title)
        seed = self.wr.get_seed()
        password = self.wr.get_password()
        numAddr = self.wr.get_num_address()

        ws = WalletService(self.wsd, self.net_type, password, load_wallet=False)
        ws.recover_wallet(seed, numAddr, self.wr.save_to_csv())

    def generate_new_address_logic(self):
        numAddr = self.wr.get_num_address()
        password = ''
        if wallet_has_password(self.net_type):
            password = self.wr.get_dec_password()

        ws = WalletService(self.wsd, self.net_type, password)
        ws.address_generate(numAddr, self.wr.save_to_csv(), self.wr.get_to_append(), 2)

    def make_payment_logic(self):
        addrIndex = self.wr.get_payment_address()
        password = ''
        if wallet_has_password(self.net_type):
            password = self.wr.get_dec_password()
        amt = self.wr.get_amount() * 10**8
        if amt < 0:
            return
        recipient = self.wr.get_recipient_address()
        attach = self.wr.get_attachment()

        ws = WalletService(self.wsd, self.net_type, password)
        ws.account_pay(addrIndex, recipient, amt, attach)

    def set_wallet_password_logic(self):
        if wallet_has_password(self.net_type):
            # has password encrypted
            old_wallet = None
            while not old_wallet:
                password = Prompt.ask('[bright_green]Please enter old password: ', password=True)
                try:
                    ws_old = WalletService(self.wsd, self.net_type, password)
                    old_wallet = ws_old.load_wallet_file()
                except WalletStorageLoadingException:
                    self.console.print('[bright_red]Invalid Password !!!')
                    continue
                if not old_wallet:
                    self.console.print('[bright_red]Invalid password for the wallet !')
                else:
                    break
            new_password = Prompt.ask('[bright_green]Please enter new password: ', password=True)
            password_again = Prompt.ask('[bright_green]Please enter new password AGAIN: ', password=True)
            if not new_password == password_again:
                self.console.input('[red]The two new password does not match!')
                return
            ws_old.set_wallet_cipher(new_password)
            ws_old.save_wallet_file(old_wallet)
        else:
            # no encryption
            password = Prompt.ask('[bright_green]Please enter new password: ', password=True)
            ws_new = WalletService(self.wsd, self.net_type, '')
            ws_new.set_wallet_cipher(password)
            old_wallet = ws_new.load_wallet_file()
            ws_new.save_wallet_file(old_wallet)
        self.console.print('[bold green]Successfully Updated !!')




import pathlib
import setuptools


HERE = pathlib.Path(__file__).parent

README = (HERE / 'README.rst').read_text()

setuptools.setup(
    name='v-cloud-market-cli-user',
    version='0.0.1',
    description='V-Cloud Market-Cli User-End',
    long_description=README,
    long_description_content_type='text/x-rst',
    url='https://github.com/virtualeconomy/v-cloud-market-cli-user.git',
    author='hvoid-build-block',
    author_email='hvoid@v.systems',
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python'
    ],
    packages=setuptools.find_packages(),
    python_requires='>=3,6'
)


import pathlib
import setuptools

HERE = pathlib.Path(__file__).parent

README = (HERE / 'README.md').read_text()


setuptools.setup(
    name='v-cloud-market-cli-commonlib',
    version='0.0.1',
    description='V-Cloud Market Cli Common Lib',
    long_description=README,
    long_description_content_type='text/markdown',
    author='hvoid-build-block',
    author_email='hvoid@v.systems',
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python'
    ],
    packages=setuptools.find_packages(),
    python_requires='>=3.6'
)


class ServiceException(Exception):
    pass

class WalletServiceException(Exception):
    pass

class WalletDecryptionException(WalletServiceException):
    pass

class WalletEncryptionException(WalletServiceException):
    pass

class WalletStorageLoadingException(WalletServiceException):
    pass

class WalletStorageSavingException(WalletServiceException):
    pass

class HttpNotFoundException(Exception):
    pass

class HttpInternalErrorException(Exception):
    pass

class HttpBadRequestException(Exception):
    pass



from v_cloud_market_cli_common.utils.service_error import (
    HttpNotFoundException,
    HttpBadRequestException
)

class ServiceCommon:

    def __init__(self):
        pass

    @staticmethod
    def validate_response(resp):
        if not isinstance(resp, dict) and not isinstance(resp, list):
            raise HttpNotFoundException
        elif 'message' in resp:
            raise HttpBadRequestException



from rich.console import Console

from market_place_cli.utils.regex import is_image_name, is_valid_path
from market_place_cli.utils.string_utils import get_unit_num
class UserServiceRequest:

    def __init__(self, console: Console):
        self.console = console

    def get_user_service_id(self) -> str:
        msg = '[bright_green]Please enter order id for detail: '
        return self.console.input(msg)

    def get_api_func(self, apis) -> (bool, str):
        api_types = ['normal', 'secret']
        while True:
            is_secret = False
            self.console.print('Please Choose API Type: ')
            msg = '[purple]1[/] -- Normal API Request\n' + \
                '[purple]2[/] -- Secret API Request\n'
            self.console.print(msg)
            choice = self._get_int_num('[bright_green]Please choose a number: ')
            if choice < 1 or choice > 2:
                continue
            is_secret = choice == 2
            while True:
                keys = list(apis[api_types[choice-1]].keys())
                api_func_msg = ''
                for index in range(len(keys)):
                    api_func_msg += f'[purple]{index+1}[/]' + ' -- ' + keys[index] + '\n'
                self.console.print(api_func_msg)
                api_func_choice = self._get_int_num('[bright_green]Please choose an API function: ')
                while api_func_choice < 1 or api_func_choice > len(keys):
                    self.console.print('[bright_red]!! Invalid Index Number !!\n')
                    api_func_choice = self._get_int_num('[bright_green]Please choose an API function: ')
                return is_secret, keys[api_func_choice-1]

    def _get_int_num(self, msg):
        try:
            choice = int(self.console.input(msg))
            return choice
        except ValueError:
            self.console.print('[bright_red]The input you entered in invalid.')
            return 0

    def get_image_info(self, check_image_func: callable) -> str:
        img_name_tag = None
        msg = '[bright_green]Please enter image name on docker hub: '
        while img_name_tag is None:
            value = self.console.input(msg)
            colon_index = value.rfind(":")
            if colon_index < 0:
                value += ":latest"
            if not is_image_name(value):
                msg = '[bright_red]Please input valid image name: '
                continue
            if not check_image_func(value):
                msg = '[bright_red]Please input image which is public read in docker hub: '
                continue
            img_name_tag = value
        return img_name_tag

    def get_ports(self, service_options: dict, provider_host: str, check_port_func: callable) -> list:
        resource_unit = service_options.get("resourceUnit", None)
        port_specification = service_options.get("portSpecification", None)
        region = service_options.get("region", None)
        if resource_unit is None or port_specification is None or region is None:
            self.console.print('[bright_red]Invalid service options.')
            return
        # get num of container-host ports pair
        try:
            host_port_num = get_unit_num(resource_unit)
        except ValueError:
                self.console.print('[bright_red]Invalid resource unit.')
        occupation_num = -1
        print(f'{host_port_num} is available in this order')
        while occupation_num < 0 or occupation_num > host_port_num:
            try:
                occupation_num = int(self.console.input(f'[bright_green]{host_port_num} host ports available, please input number of host port you want: '))
            except ValueError:
                self.console.print(f'[bright_red]Invalid input, please input number from 0 to {host_port_num}.')

        port_configs = []
        self.console.print(f'{occupation_num} port(s) will be set by user')
        for x in range(occupation_num):
            self.console.print(f'Container or host port {x + 1}:')
            container_port = self.get_container_port(port_configs)
            port_config = {
                "containerPort": container_port
            }
            if port_specification == "User Specified Service Port":
                host_port = self.get_host_port(region, provider_host, port_configs, check_port_func)
                port_config["hostPort"] = host_port
            port_configs.append(port_config)
        return port_configs

    def get_container_port(self, ports_configs: list) -> int:
        value = 0
        while value <= 0 or value > 65535:
            try:
                value = int(self.console.input('Please input container port: '))
                # check repetition
                for config in ports_configs:
                    if config["containerPort"] == value:
                        self.console.print('[bright_red]Container port cannot be repetitive! Please use another port.')
                        value = 0
            except ValueError:
                self.console.print('[bright_red]Invalid port number.')
                value = 0
        return value

    def get_host_port(self, region: str, provider_host: str, ports_configs: list, check_port_func: callable) -> int:
        value = 0
        msg = '[bright_green]Please input host port: '
        is_valid = False
        while not is_valid:
            try:
                value = int(self.console.input(msg))
                is_valid = value <= 0 or value > 65535
                if is_valid:
                    msg = '[bright_red]Valid port number range in 1 ~ 65535, please use valid number: '
                    continue
                is_duplcated = False
                for config in ports_configs:
                    is_duplcated = (config["hostPort"] == value)
                if is_duplcated:
                    self.console.print('[bright_red]Host port cannot be repetitive! Please use another port.')
                    continue
                resp = check_port_func([value], region, provider_host)
                status = resp.get("status", None)
                is_valid = status is None or status == "NotOccupied"
                if not is_valid:
                    msg = '[bright_red]Inputted port has been occupied, please change port number: '
            except ValueError as e:
                print(e)
                msg = '[bright_red]Please input valid integer:'
                value = 0
        return value

    def get_mount_path(self) -> str:
        msg = "[bright_green]Please input directory to mount data: "
        mount_path = self.console.input(msg)
        while not is_valid_path(mount_path):
            mount_path = self.console.input("[bright_red]Mount path is invalid: ")
        return mount_path



from rich.console import Console

class OrderRequest:

    def __init__(self, console: Console):
        self.console = console

    def get_order_id(self) -> str:
        msg = 'Please enter the order id: '
        return self.console.input(msg)


from Crypto.Cipher import AES
from Crypto.Hash import keccak, BLAKE2b, SHA256
from Crypto import Random
from base64 import b64encode, b64decode
from base58 import b58encode
from hashlib import pbkdf2_hmac

from v_cloud_market_cli_common.config.wallet_config import WORD_LIST
from .vsyschain.error import throw_error
from .service_error import WalletDecryptionException, WalletEncryptionException

import axolotl_curve25519 as curve

class WalletCipher:

    def __init__(self, password):
        self.keyLength = 256
        self.hashingIterations = 9999
        self.encoding = "utf-8"
        self.keySalt = "0ba950e1-828b-4bae-9e06-faf078eb33ec"
        self.key = self.__hash_password(password)

    def encrypt(self, infoStr):
        try:
            raw = self.__pad(infoStr)
            iv = Random.new().read(AES.block_size)
            cipher = AES.new(self.key[:32], AES.MODE_CBC, iv)
            return str(b64encode(iv + cipher.encrypt(raw)), encoding=self.encoding)
        except:
            msg = 'Fail to encrypt info string'
            throw_error(msg, WalletEncryptionException)

    def decrypt(self, raw):
        if raw is None:
            return None
        try:
            enc = b64decode(raw)
            iv = enc[:16]
            enc = enc[16:]
            cipher = AES.new(self.key[:32], AES.MODE_CBC, iv)
            return str(cipher.decrypt(enc), encoding = self.encoding).replace("\0", "")
        except:
            msg = "Failed to decrypt info string"
            throw_error(msg, WalletDecryptionException)

    def __hash_password(self, password):
        return pbkdf2_hmac("sha512", password.encode("utf-8"), self.keySalt.encode(self.encoding), self.hashingIterations, self.keyLength)

    def __pad(self, inStr):
        while len(inStr.encode(self.encoding)) % 16 != 0:
            inStr += "\0"
        return inStr.encode(self.encoding)

    @staticmethod
    def generate_phrase():
        wordCount = 2048
        phrase = ""
        ind_1 = ind_2 = ind_3 = 0
        x = int(0)
        for i in range(5):
            r = Random.get_random_bytes(4)
            x = (r[3] & 0xff) + \
                ((r[2] & 0xff) << 8) + \
                ((r[1] & 0xff) << 16) + \
                ((r[0] & 0xff) << 24)
            ind_1 = int(x % wordCount)
            ind_2 = int(((int(x / wordCount) >> 0) + ind_1) % wordCount)
            ind_3 = int(((int((int(x / wordCount) >> 0) / wordCount) >> 0) + ind_2) % wordCount)
            phrase += WORD_LIST[ind_1] + " "
            phrase += WORD_LIST[ind_2] + " "
            phrase += WORD_LIST[ind_3] + " "
        return phrase.strip(" ")

    @staticmethod
    def hash_chain(noncedSeed):
        """
        noncedSeed: bytes object
        return: bytes object
        """
        keccak_hash = keccak.new(digest_bits=256)
        blake_hash = BLAKE2b.new(digest_bits=256)
        tmp = blake_hash.update(noncedSeed).digest()
        result = keccak_hash.update(tmp).digest()
        return result

    @staticmethod
    def generate_keypair(seed):
        """
        seed: bytes object
        return: bytes object
        """
        sha256 = SHA256.new()
        sha256.update(seed)
        privateKey = curve.generatePrivateKey(sha256.digest())
        publicKey = curve.generatePublicKey(privateKey)
        return privateKey, publicKey

    @staticmethod
    def generate_address(pubKey, net="T"):
        """
        pubKey: bytes object
        return: bytes object
        """
        addrVer = bytes([5])
        chainId = bytes(net, "ascii")
        unchecksumedAddress = addrVer + chainId + WalletCipher.hash_chain(pubKey)[:20]
        address = b58encode(unchecksumedAddress + WalletCipher.hash_chain(unchecksumedAddress)[:4])
        return address

    @staticmethod
    def generate_account_info(seed, net='T', nonce=0):
        accSeed = WalletCipher.hash_chain((str(nonce) + seed).encode('utf-8'))
        priv, pub = WalletCipher.generate_keypair(accSeed)
        addr = WalletCipher.generate_address(pub, net).decode('utf-8')
        return {'nonce': nonce, 'accSeed': accSeed, 'priv': priv, 'pub': pub, 'addr': addr}



# data is a list of dict
def max_length(data, key):
    maxVal = 0
    for item in data:
        n = len(str(item[key]))
        if maxVal < n:
            maxVal = n
    return maxVal if maxVal > len(key) else len(key)



import hashlib
import pyblake2
import axolotl_curve25519 as curve
import base58
import os
from math import log
from operator import xor
from copy import deepcopy
import functools


if bytes == str:  # python2
    def str2bytes(s): return s
    def bytes2str(b): return b
    def str2list(s): return [ord(c) for c in s]
    def list2bytes(s): return ''.join(s)
    no_return_bytes = [chr(0)]
    def to_hex(s): return ''.join('{:02x}'.format(x) for x in s)
else:  # python3
    def str2bytes(s): return s.encode('latin-1')
    def bytes2str(b): return ''.join(map(chr, b))
    def str2list(s): return [c for c in s]
    def list2bytes(s): return b''.join(s)
    no_return_bytes = bytes('', encoding='utf-8')
    def to_hex(s): return s.hex()


RoundConstants = [
    0x0000000000000001, 0x0000000000008082, 0x800000000000808A, 0x8000000080008000,
    0x000000000000808B, 0x0000000080000001, 0x8000000080008081, 0x8000000000008009,
    0x000000000000008A, 0x0000000000000088, 0x0000000080008009, 0x000000008000000A,
    0x000000008000808B, 0x800000000000008B, 0x8000000000008089, 0x8000000000008003,
    0x8000000000008002, 0x8000000000000080, 0x000000000000800A, 0x800000008000000A,
    0x8000000080008081, 0x8000000000008080, 0x0000000080000001, 0x8000000080008008
]

RotationConstants = [
    [0, 1, 62, 28, 27, ],
    [36, 44, 6, 55, 20, ],
    [3, 10, 43, 25, 39, ],
    [41, 45, 15, 21, 8, ],
    [18, 2, 61, 56, 14, ]
]

Masks = [(1 << i) - 1 for i in range(65)]


def bits2bytes(x):
    return (int(x) + 7) / 8


def rol(value, left, bits):
    top = value >> (bits - left)
    bot = (value & Masks[bits - left]) << left
    return bot | top


def ror(value, right, bits):
    top = value >> right
    bot = (value & Masks[right]) << (bits - right)
    return bot | top


def multirate_padding(used_bytes, align_bytes):
    padlen = align_bytes - used_bytes
    if padlen == 0:
        padlen = align_bytes
    # note: padding done in 'internal bit ordering', wherein LSB is leftmost
    if padlen == 1:
        return [0x81]
    else:
        return [0x01] + ([0x00] * (int(padlen) - 2)) + [0x80]


def keccak_f(state):
    def round(A, RC):
        W, H = state.W, state.H
        rangeW, rangeH = state.rangeW, state.rangeH
        lanew = state.lanew
        zero = state.zero

        # theta
        C = [functools.reduce(xor, A[x]) for x in rangeW]
        D = [0] * W
        for x in rangeW:
            D[x] = C[(x - 1) % W] ^ rol(C[(x + 1) % W], 1, lanew)
            for y in rangeH:
                A[x][y] ^= D[x]

        # rho and pi
        B = zero()
        for x in rangeW:
            for y in rangeH:
                B[y % W][(2 * x + 3 * y) % H] = rol(A[x][y],
                                                    RotationConstants[y][x], lanew)

        # chi
        for x in rangeW:
            for y in rangeH:
                A[x][y] = B[x][y] ^ ((~ B[(x + 1) % W][y]) & B[(x + 2) % W][y])

        # iota
        A[0][0] ^= RC

    l = int(log(state.lanew, 2))
    nr = 12 + 2 * l

    for ir in range(nr):
        round(state.s, RoundConstants[ir])


class KeccakState(object):
    W = 5
    H = 5

    rangeW = range(W)
    rangeH = range(H)

    @staticmethod
    def zero():
        return [[0] * KeccakState.W for x in KeccakState.rangeH]

    @staticmethod
    def format(st):
        rows = []

        def fmt(x):
            return '%016x' % x

        for y in KeccakState.rangeH:
            row = []
            for x in KeccakState.rangeW:
                row.append(fmt(st[x][y]))
            rows.append(' '.join(row))
        return '\n'.join(rows)

    @staticmethod
    def lane2bytes(s, w):
        o = []
        for b in range(0, w, 8):
            o.append((s >> b) & 0xff)
        return o

    @staticmethod
    def bytes2lane(bb):
        r = 0
        for b in reversed(bb):
            r = r << 8 | b
        return r

    def __init__(self, bitrate, b):
        self.bitrate = bitrate
        self.b = b

        # only byte-aligned
        assert self.bitrate % 8 == 0
        self.bitrate_bytes = bits2bytes(self.bitrate)

        assert self.b % 25 == 0
        self.lanew = self.b // 25

        self.s = KeccakState.zero()

    def __str__(self):
        return KeccakState.format(self.s)

    def absorb(self, bb):
        assert len(bb) == self.bitrate_bytes

        bb += [0] * int(bits2bytes(self.b - self.bitrate))
        i = 0

        for y in self.rangeH:
            for x in self.rangeW:
                self.s[x][y] ^= KeccakState.bytes2lane(bb[i:i + 8])
                i += 8

    def squeeze(self):
        return self.get_bytes()[:self.bitrate_bytes]

    def get_bytes(self):
        out = [0] * int(bits2bytes(self.b))
        i = 0
        for y in self.rangeH:
            for x in self.rangeW:
                v = KeccakState.lane2bytes(self.s[x][y], self.lanew)
                out[i:i + 8] = v
                i += 8
        return out

    def set_bytes(self, bb):
        i = 0
        for y in self.rangeH:
            for x in self.rangeW:
                self.s[x][y] = KeccakState.bytes2lane(bb[i:i + 8])
                i += 8


class KeccakSponge(object):
    def __init__(self, bitrate, width, padfn, permfn):
        self.state = KeccakState(bitrate, width)
        self.padfn = padfn
        self.permfn = permfn
        self.buffer = []

    def copy(self):
        return deepcopy(self)

    def absorb_block(self, bb):
        self.state.bitrate_bytes = int(self.state.bitrate_bytes)
        assert len(bb) == self.state.bitrate_bytes
        self.state.absorb(bb)
        self.permfn(self.state)

    def absorb(self, s):
        self.buffer = str2list(s)

        while len(self.buffer) >= self.state.bitrate_bytes:
            self.absorb_block(self.buffer[:self.state.bitrate_bytes])
            self.buffer = self.buffer[self.state.bitrate_bytes:]

    def absorb_final(self):
        padded = self.buffer + \
            self.padfn(len(self.buffer), self.state.bitrate_bytes)
        self.absorb_block(padded)
        self.buffer = []

    def squeeze_once(self):
        rc = self.state.squeeze()
        self.permfn(self.state)
        return rc

    def squeeze(self, l):
        Z = self.squeeze_once()
        while len(Z) < l:
            Z += self.squeeze_once()
        return Z[:int(l)]


class KeccakHash(object):
    def __init__(self):
        bitrate_bits = 1088
        capacity_bits = 512
        output_bits = 256
        self.sponge = KeccakSponge(bitrate_bits, bitrate_bits + capacity_bits,
                                   multirate_padding,
                                   keccak_f)
        self.digest_size = bits2bytes(output_bits)
        self.block_size = bits2bytes(bitrate_bits)

    def __repr__(self):
        inf = (self.sponge.state.bitrate,
               self.sponge.state.b - self.sponge.state.bitrate,
               self.digest_size * 8)
        return '<KeccakHash with r=%d, c=%d, image=%d>' % inf

    def digest(self, s):
        self.sponge.absorb(s)
        finalised = self.sponge.copy()
        finalised.absorb_final()
        digest = finalised.squeeze(self.digest_size)
        return ''.join(map(chr, digest))


keccak256 = KeccakHash()


def sha256(s):
    return hashlib.sha256(str2bytes(s)).digest()


def hashChain(s):
    a = pyblake2.blake2b(s, digest_size=32).digest()
    b = keccak256.digest(a)
    return b


def sign(privateKey, message):
    random64 = os.urandom(64)

    return base58.b58encode(curve.calculateSignature(random64, base58.b58decode(privateKey), message))


def id(message):
    return base58.b58encode(hashlib.sha256(message).digest())



from rich.console import Console
from rich.table import Table
from rich.panel import Panel
from rich.measure import Measurement
from rich import box
from base58 import b58encode


class WalletServiceDisplay:

    def __init__(self, console: Console):
        self.console = console

    def show_address(self, accInfoList: [dict]) -> None:
        self.console.clear()
        table = Table()
        table.title = 'Wallet Address Information'
        table.add_column('Address', justify='center')
        table.add_column('Account Seed', justify='center')
        table.add_column('Private Key', justify='center')
        table.add_column('Public Key', justify='center')
        table.columns[0].style = 'green'
        table.columns[0].header_style = 'bold green'
        table.columns[1].style = 'magenta'
        table.columns[1].header_style = 'bold magenta'
        table.columns[2].style = 'red'
        table.columns[2].header_style = 'bold red'
        table.columns[3].style = 'bright_yellow'
        table.columns[3].header_style = 'bold bright_yellow'
        table.row_styles = ['none', 'dim']
        rows = self._form_address_rows(accInfoList)
        for row in rows:
            table.add_row(*row)

        balanceTable = Table()
        balanceTable.row_styles = ['none', 'dim']
        balanceTable.title = 'Wallet Address Balance Information'
        balanceTable.add_column('Address', justify='center')
        balanceTable.columns[0].style = 'dark_sea_green4'
        balanceTable.columns[0].header_style = 'bold dark_sea_green4'
        if 'balance' in accInfoList[0]:
            balanceTable.add_column('Balance', '[u]412,000,000 VSYS', justify='right')
            balanceTable.columns[1].style = 'bright_green'
            balanceTable.columns[1].header_style = 'bold bright_green'
        elif 'balanceDetail' in accInfoList[0]:
            balanceTable.add_column('Regular Balance', '[u]412,000,000', justify='right')
            balanceTable.add_column('Minting Average', '[u]412,000,000', justify='right')
            balanceTable.add_column('Available Balance', '[u]412,000,000', justify='right')
            balanceTable.add_column('Effective Balance', '[u]412,000,000', justify='right')
            balanceTable.columns[1].style = 'bright_yellow'
            balanceTable.columns[1].header_style = 'bold bright_yellow'
            balanceTable.columns[2].style = 'bright_red'
            balanceTable.columns[2].header_style = 'bold bright_red'
            balanceTable.columns[3].style = 'bright_green'
            balanceTable.columns[3].header_style = 'bold bright_green'
            balanceTable.columns[4].style = 'cyan'
            balanceTable.columns[4].header_style = 'bold cyan'

        bRows = self._form_balance_rows(accInfoList)
        for row in bRows:
            balanceTable.add_row(*row)

        table.box = box.SIMPLE_HEAD
        balanceTable.box = box.SIMPLE_HEAD
        table_w = Measurement.get(self.console, table).maximum
        balanceTable.width = table_w
        self.console.print(table, justify='left')
        self.console.print(balanceTable, justify='left')
        self.console.input('Press ENTER to continue...')

    def show_wallet(self, wallet_data: dict) -> None:
        self.console.clear()
        if not wallet_data or len(wallet_data) == 0:
            self.console.print('[dark_red]! No Wallet Data File ![/]')
            return
        table = Table()
        table.add_column('Agent', no_wrap=True, justify='center')
        table.add_column('Seed', justify='center')
        table.add_column('Last Nonce #', justify='center')
        row = [wallet_data.agent, wallet_data.seed, str(wallet_data.nonce)]
        table.add_row(*row)
        self.console.print(table, justify='center')

        accSeedTable = Table()
        accSeedTable.add_column('Account Seed', no_wrap=True, justify='left')
        for accSeed in wallet_data.accountSeeds:
            accSeedTable.add_row(accSeed)
        self.console.print(accSeedTable, justify='center')

    def show_account_pay(self, recipient: str, amount: float) -> None:
        self.console.clear()
        self.console.print('[dark_sea_green4]=[/]' * 25)
        self.console.print('[dark_sea_green4]+[/] Successful Payment! [dark_sea_green4]+[/]')
        self.console.print('[dark_sea_green4]=[/]' * 25)
        self.console.print('Recipient:        : ' + recipient)
        self.console.print('Amount            : ' + str(amount))
        self.console.input('Press ENTER to continue...')

    def show_seed(self, seed: str) -> None:
        self.console.clear()
        msg = '[bright_red]IMPORTANT - COPY OR MEMORIZE THE SEED PHRASE BELOW FOR KEY RECOVERY!!!\n'
        msg += '[bright_green]seed              :' + seed
        p = Panel(msg, box=box.DOUBLE_EDGE, expand=False)
        self.console.print(p)
        self.console.input('Press ENTER to continue...')

    def _form_address_rows(self, accInfoList: [dict]) -> [[str]]:
        rows = []
        for accInfo in accInfoList:
            row = []
            row.append(accInfo['addr'])
            row.append(b58encode(accInfo['accSeed']).decode('utf-8'))
            row.append(b58encode(accInfo['priv']).decode('utf-8'))
            row.append(b58encode(accInfo['pub']).decode('utf-8'))
            rows.append(row)
        return rows

    def _form_balance_rows(self, accInfoList: [dict]) -> [[str]]:
        rows = []
        for accInfo in accInfoList:
            row = []
            row.append(accInfo['addr'])
            rows.append(self._add_balance_info(accInfo, row))
        return rows

    def _add_balance_info(self, accInfo: dict, row: [str]) -> [str]:
        if 'balance' in accInfo:
            row.append(str(accInfo['balance']))
        elif 'balanceDetail' in accInfo:
            d = accInfo['balanceDetail']
            row.append(str(d['regular']))
            row.append(str(d['mintingAverage']))
            row.append(str(d['available']))
            row.append(str(d['effective']))
        return row





import logging
import sys
from .setting import THROW_EXCEPTION_ON_ERROR

class PyVException(Exception):
    pass

class InvalidParameterException(PyVException):
    pass

class InvalidAddressException(PyVException):
    pass

class NetworkException(PyVException):
    pass

class MissingPrivateKeyException(PyVException):
    pass

class InsufficientBalanceException(PyVException):
    pass

def set_throw_on_error(throw=True):
    global THROW_EXCEPTION_ON_ERROR
    THROW_EXCEPTION_ON_ERROR = throw

def throw_error(msg, exception=Exception):
    logging.error(msg)
    if THROW_EXCEPTION_ON_ERROR:
        raise exception(msg)



from datetime import datetime
from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from rich import box
from rich.measure import Measurement

from .display_common import utc_to_local
from .table import display_table


class UserServiceDisplay:

    def __init__(self, console: Console):
        self.console = console

    def show_user_service_page(self, title: str, infoList: [dict]):
        self.console.clear()
        table = Table(show_header=True)
        table.title = '[bold bright_magenta]' + title
        table.box = box.ROUNDED
        table.add_column('Order ID', justify='center')
        table.add_column('Service Type', justify='center')
        table.add_column('Creation Time', justify='center')
        table.add_column('Duration (HOUR)', justify='center')
        table.add_column('Expiration Time', justify='center')
        for col in table.columns:
            col.header_style = 'magenta'

        rows = self._form_user_service_rows(infoList)
        for row in rows:
            table.add_row(*row)
        self.console.print(table, justify='center')
        return Measurement.get(self.console, table).maximum

    def show_user_service_detail(self, info):
        table = Table(show_header=True)
        table.box = box.ROUNDED
        table.title = '[bold bright_magenta]' + info['id'] + '\nUser Service Details'
        table.add_column('Service Type', justify='center')
        table.add_column('Service Options', justify='left')
        for col in table.columns:
            col.header_style = 'magenta'

        row = [info['service'], self._form_service_options(info['serviceOptions'])]
        table.add_row(*row)
        self.console.print(table, justify='center')
        self.console.input('Press ENTER to continue...')

    def display_user_service(self, info, secret: str = ''):
        msg = '[bold magenta]ID:[/] ' + info['userServiceID'] + '\n' + \
              '[bold magenta]Service:[/] ' + info['service'] + '\n' + \
              '[bold magenta]Service ID:[/] ' + info['serviceID'] + '\n' + \
              '[bold magenta]Address:[/] ' + info['address'] + '\n' + \
              '[bold magenta]Status:[/] ' + info['status'] + '\n' + \
              '[bold magenta]Service Active Timestamp:[/] ' + self.to_local_time_str(info['serviceActiveTS']) + '\n'

        if info['status'] in ['ServiceRunning', 'ServiceDone', 'ServiceAbort']:
            time_str = self.to_local_time_str(info['serviceRunningTS'])
            msg += '[bold magenta]Service Running Timestamp:[/] ' + time_str + '\n'
        if info['status'] == 'ServiceDone':
            time_str = self.to_local_time_str(info['serviceDoneTS'])
            msg += '[bold magenta]Service Done Timestamp:[/] ' + time_str + '\n'
        if info['status'] == 'ServiceAbort':
            time_str = self.to_local_time_str(info['serviceAbortTS'])
            msg += '[bold magenta]Service Abort Timestamp:[/] ' + time_str + '\n'

        msg += '[bold magenta]Service Options:[/] \n' + self._form_service_options(info['serviceOptions'])

        if secret:
            msg += '[bold magenta]Secret Info:[/] \n' + secret + '\n'
        self.console.print(Panel.fit(msg))
        self.console.input('Press ENTER to continue...')

    def display_service_status(self,  headers: list, status_list: list):
        self.console.clear()
        title = '[bold bright_magenta] User Service Status'
        try:
            display_table(self.console, title, headers, status_list)
        except Exception as err:
            self.console.print(err)

    def _form_user_service_rows(self, infoList: [dict]) -> [[str]]:
        rows = []
        for info in infoList:
            row = [info['id'], info['service']]
            cTime = self.to_local_time_str(info['serviceActivateTS'])
            if int(info['endAt']) == 0:
                eTime = 'xxxxxx'
            else:
                eTime = self.to_local_time_str(info['endAt'])
            row.extend([cTime, str(info['duration']), eTime])
            rows.append(row)
        return rows

    def _form_service_options(self, serviceOptions) -> str:
        service_opts = ''
        if not serviceOptions or len(serviceOptions) == 0:
            return service_opts
        for opt_key in serviceOptions.keys():
            service_opts += '[bright_green]' + opt_key + ':[/]\n'
            service_opts += ' ' * 4 + serviceOptions[opt_key] + '\n'
        return service_opts

    def to_local_time_str(self, ts: str):
        return utc_to_local(datetime.utcfromtimestamp(int(ts))).strftime('%Y-%m-%d %H:%M:%S')




from setuptools import setup, find_packages
with open("README.md", "r") as f:
    long_description = f.read()

with open('requirements.txt') as f:
    requirements = f.readlines()

setup(
    name='v_cloud_market_cli_user',
    version='0.1.9',
    description='V Cloud Market Command Tool For User.',
    long_description=long_description,
    long_description_context_type='text/markdown',
    keywords=['vcloud-market-cli-user'],
    url='https://github.com/virtualeconomy/v-cloud-market-cli-user',
    author='hvoid-build-block',
    license='MIT',
    packages=find_packages(),
    python_requires='>=3.5',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python'
    ],
    entry_points={
        'console_scripts': [
            'vcloud = market_place_cli.main_logic:start'
        ]
    },
    install_requires=requirements,
    zip_safe=False,
)


from rich.console import Console
from rich.panel import Panel
from rich.measure import Measurement
from rich.table import Table
from rich import box


class MarketServiceDisplay:

    def __init__(self, console: Console):
        self.console = console

    def display_service_page(self, result: dict):
        self.console.clear()
        table = Table(show_header=True, header_style='bright_magenta')
        table.title = '[bold magenta]Service Type Information'
        table.box = box.ROUNDED
        table.add_column('Service ID')
        table.add_column('Service Provider')
        table.add_column('Service Name')
        table.add_column('Service Options', no_wrap=True, justify='full')
        table.add_column('Available Service API', no_wrap=True)
        table.add_column('Refundable')
        table.add_column('Duration To Price', no_wrap=True)
        table.add_column('Description')

        for row in self._form_service_type_rows(result['list']):
            table.add_row(*row)
        self.console.print(table, justify='center')
        return Measurement.get(self.console, table).maximum

    def display_category_page(self, result: dict):
        self.console.clear()
        table = Table(show_header=True, header_style='bright_magenta')
        table.title = '[bold magenta]Service Category Information'
        table.box = box.ROUNDED
        table.add_column('Service Category ID')
        table.add_column('Provider')
        table.add_column('Category Name')
        table.add_column('Service Type')
        table.add_column('Service Options')
        table.add_column('Description')

        for row in self._form_category_rows(result['list']):
            table.add_row(*row)
        self.console.print(table, justify='center')
        return Measurement.get(self.console, table).maximum

    def display_provider_page(self, result: dict):
        self.console.clear()
        table = Table(show_header=True, header_style='bright_magenta')
        table.title = '[bold magenta]Service Provider Information'
        table.box = box.ROUNDED
        table.add_column('Provider Name')
        table.add_column('Wallet Address')
        table.add_column('Provided Service Category Name')

        for row in self._form_provider_rows(result['list']):
            table.add_row(*row)
        self.console.print(table, justify='center')
        return Measurement.get(self.console, table).maximum

    def _form_provider_rows(self, providers):
        rows = []
        for provider in providers:
            row = [provider['name'], provider['walletAddress']]
            categories = ''
            for c in provider['category2ID'].keys():
                categories += c + '\n'
            row.append(categories)
            rows.append(row)
        return rows

    def _form_category_rows(self, categories):
        rows = []
        for category in categories:
            row = [category['categoryID'], category['provider'], category['name']]
            service_type = ''
            service_opts = ''

            if category['name2ID']:
                for sType in category['name2ID'].keys():
                    service_type += sType + '\n'

            if category['serviceOptions']:
                service_opts = self._form_service_options(category['serviceOptions'])
            row.extend([service_type, service_opts, category['description']])
            rows.append(row)
        return rows

    def _form_service_type_rows(self, service_types):
        rows = []
        for service_type in service_types:
            row = [service_type['id'], service_type['provider'], service_type['name'],
                   self._form_service_options(service_type['serviceOptions']),
                   self._form_service_api((service_type['serviceAPI'])),  str(service_type['refundable']),
                   self._form_duration_to_price(service_type['durationToPrice']), service_type['description']]
            rows.append(row)
        return rows

    def _form_service_options(self, serviceOptions) -> str:
        service_opts = ''
        if not serviceOptions or len(serviceOptions) == 0:
            return service_opts
        for opt_key in serviceOptions.keys():
            service_opts += '[bright_green]' + opt_key + ':[/]\n'
            for opt in serviceOptions[opt_key]:
                service_opts += ' ' * 4 + opt + '\n'
        return service_opts

    def _form_duration_to_price(self, duration_to_prices) -> str:
        result = ''
        for ps in duration_to_prices:
            tmp = ''
            tmp += '[bold bright_green]Price: ' + str(ps['price']) + '\n'
            tmp += '[bold bright_green]Charging Service Options: \n'
            for opt in ps['chargingOptions']:
                tmp += '\t[light_cyan1]' + opt + ': ' + ps['chargingOptions'][opt] + '\n'
            tmp += '[bold bright_green]Price Factor for Different Time Period:\n'
            for d in ps['duration']:
                tmp += '\t[light_cyan1]' + str(d) + ' hour: ' + str(ps['duration'][d]) + '\n'
            result += tmp + '\n'
            result += "=" * Measurement.get(self.console, result).maximum + '\n'
        return result

    def _form_service_api(self, service_apis):
        result = '[bold magenta]API Types\n'
        for api in service_apis:
            if isinstance(service_apis[api], dict):
                result += f'[bold bright_green]{api} API:\n'
                for key in service_apis[api]:
                    result += ' ' * 4 + f'[bold cyan1]{key}\n'
        return result

    def _form_name_to_id(self, name2ID):
        result = ''
        for k in name2ID:
            row = f'[bright_green]{k}[/]: {name2ID[k]}'
            result += row + '\n'
        return result

    def display_order_brief(self, info):
        msg = '[bold bright_magenta]Order ID[/]: ' + info['id'] + '\n' + \
              '[bold bright_magenta]Recipient[/]: ' + info['recipient'] + '\n' + \
              '[bold bright_magenta]Amount[/]: ' + str(info['amount'])
        self.console.print(Panel.fit(msg,
                                     title='[bold bright_green]! New Order Created !',
                                     title_align='center'),
                           justify='center')
        self.console.input('Press ENTER to continue...')

    def display_service_type(self, info):
        msg = '[bold bright_magenta]Service Type ID:[/] ' + info['id'] + '\n' + \
              '[bold bright_magenta]Service Name:[/] ' + info['name'] + '\n' + \
              '[bold bright_magenta]Service Category:[/] ' + info['category'] + '\n' + \
              '[bold bright_magenta]Provider:[/] ' + info['provider'] + '\n' + \
              '[bold bright_magenta]Refundable:[/] ' + str(info['refundable']) + '\n' + \
              '[bold bright_magenta]Service Options:[/] \n' + self._form_service_options(info['serviceOptions']) + '\n' + \
              '[bold bright_magenta]Duration To Price:[/] \n' + self._form_duration_to_price(info['durationToPrice'])
        self.console.print(Panel.fit(msg,
                                     title='[bold bright_green]Service Type Information',
                                     title_align='center'))
        self.console.input('Press ENTER to continue...')

    def display_service_category(self, info):
        msg = '[bold bright_magenta]Category ID:[/] ' + info['categoryID'] + '\n' + \
              '[bold bright_magenta]Category Name:[/] ' + info['name'] + '\n' + \
              '[bold bright_magenta]Provider:[/] ' + info['provider'] + '\n' + \
              '[bold bright_magenta]Description:[/] ' + info['description'] + '\n' + \
              '[bold bright_magenta]Service Options:[/]\n' + self._form_service_options(info['serviceOptions']) + '\n' + \
              '[bold bright_magenta]Service Type To ID:[/]\n' + self._form_name_to_id(info['name2ID'])
        self.console.print(Panel.fit(msg,
                                     title='[bold bright_green]Service Category Information',
                                     title_align='center'))
        self.console.input('Press ENTER to continue...')

    def display_provider(self, info):
        msg = '[bold bright_magenta]Name:[/] ' + info['name'] + '\n' + \
              '[bold bright_magenta]Wallet Address:[/] ' + info['walletAddress'] + '\n' + \
              '[bold bright_magenta]Service Category To ID:[/]\n' + self._form_name_to_id(info['category2ID'])
        self.console.print(Panel.fit(msg,
                                     title='[bold bright_green]Service Provider Information',
                                     title_align='center'))
        self.console.input('Press ENTER to continue...')
